@@ -78,23 +78,9 @@ GST_DEBUG_CATEGORY_EXTERN (alsa_debug);
 
 #define GST_ALSA_MIN_RATE	8000
 #define GST_ALSA_MAX_RATE	192000
-#define GST_ALSA_MIN_PERIOD_CNT	0
-#define GST_ALSA_MAX_PERIOD_CNT	64
-#define GST_ALSA_MIN_PERIOD_SZ	2
-#define GST_ALSA_MAX_PERIOD_SZ	8192
-#define GST_ALSA_MIN_BUFFER_SZ	GST_ALSA_MIN_PERIOD_CNT*GST_ALSA_MIN_PERIOD_SZ
-#define GST_ALSA_MAX_BUFFER_SZ	65536
 #define GST_ALSA_MAX_TRACKS	64 /* we don't support more than 64 tracks */
 #define GST_ALSA_MAX_CHANNELS	32 /* tracks can have up to 32 channels */
 
-/* a few alsa functions return an int value and a 'direction', -1, 0 or +1
-  0 = exact value
- -1 = real value is up to 1 before given value
- +1 = real value is up to 1 after given value
-*/
-#define GST_ALSA_DIR_MIN(i) ((i == 1) ? 1 : 0)
-#define GST_ALSA_DIR_MAX(i) ((i ==-1) ?-1 : 0)
-
 /* Mono is 1 channel ; the 5.1 standard is 6 channels. The value for
    GST_ALSA_MAX_CHANNELS comes from alsa/mixer.h. */
 
@@ -161,27 +147,20 @@ struct _GstAlsa {
   GstAlsaFormat *		format;		/* NULL if undefined */
   gboolean			mmap; 		/* use mmap transmit (fast) or read/write (sloooow) */
   GstAlsaTransmitFunction	transmit;
-  GstCaps *			cached_caps;	/* we cache caps to speed up get_caps */
 
   /* latency / performance parameters */
-  snd_pcm_uframes_t		desired_period_size;	/* as set by user */
   snd_pcm_uframes_t		period_size;
-  unsigned int			desired_period_count;	/* as set by user */
   unsigned int			period_count;
 
   gboolean			autorecover;
 
-  gboolean                      advanced_probing;
-
   /* clocking */
   GstAlsaClock *		clock;		/* our provided clock */
-  GstClock *			ext_clock;	/* externally set clock. */
-  snd_pcm_uframes_t		played; 	/* samples transmitted since last sync 
+  snd_pcm_uframes_t		transmitted; 	/* samples transmitted since last sync 
 						   This thing actually is our master clock.
 						   We will event insert silent samples or
 						   drop some to sync to incoming timestamps.
 						 */
-  snd_pcm_uframes_t		captured;
   GstClockTime			max_discont;	/* max difference between current
   						   playback timestamp and buffers timestamps
 						 */
@@ -211,7 +190,6 @@ GstCaps *		gst_alsa_fixate 	(GstPad *		pad,
 GstCaps *		gst_alsa_caps		(snd_pcm_format_t	format,
 						 gint			rate,
 						 gint			channels);
-gboolean                gst_alsa_set_default_format (GstAlsa * this);
 
 /* audio processing functions */
 inline snd_pcm_sframes_t	gst_alsa_update_avail	(GstAlsa * this);
