@@ -63,10 +63,21 @@ enum
   ARG_0,
   ARG_CLAMP_LUMA,
   ARG_CLAMP_CHROMA,
-  ARG_QUALITY,
-  /* FILL ME */
+  ARG_QUALITY
+      /* FILL ME */
 };
 
+const gint qualities[] = {
+  DV_QUALITY_DC,
+  DV_QUALITY_AC_1,
+  DV_QUALITY_AC_2,
+  DV_QUALITY_DC | DV_QUALITY_COLOR,
+  DV_QUALITY_AC_1 | DV_QUALITY_COLOR,
+  DV_QUALITY_AC_2 | DV_QUALITY_COLOR
+};
+
+#define DV_QUALITY_DEFAULT 5
+
 /* The PadFactory structures describe what pads the element has or
  * can have.  They can be quite complex, but for this dvdec plugin
  * they are rather simple.
@@ -93,9 +104,9 @@ static GstStaticPadTemplate video_src_temp = GST_STATIC_PAD_TEMPLATE ("video",
         "bpp = (int) 32, "
         "depth = (int) 32, "
         "endianness = (int) " G_STRINGIFY (G_BIG_ENDIAN) ", "
-        "red_mask =   (int) 0x000000ff, "
+        "red_mask =   (int) 0x00ff0000, "
         "green_mask = (int) 0x0000ff00, "
-        "blue_mask =  (int) 0x00ff0000, "
+        "blue_mask =  (int) 0x000000ff, "
         "width = (int) 720, "
         "height = (int) { "
         G_STRINGIFY (NTSC_HEIGHT) ", " G_STRINGIFY (PAL_HEIGHT)
@@ -107,9 +118,9 @@ static GstStaticPadTemplate video_src_temp = GST_STATIC_PAD_TEMPLATE ("video",
         "bpp = (int) 24, "
         "depth = (int) 24, "
         "endianness = (int) " G_STRINGIFY (G_BIG_ENDIAN) ", "
-        "red_mask =   (int) 0x000000ff, "
+        "red_mask =   (int) 0x00ff0000, "
         "green_mask = (int) 0x0000ff00, "
-        "blue_mask =  (int) 0x00ff0000, "
+        "blue_mask =  (int) 0x000000ff, "
         "width = (int) 720, "
         "height = (int) { "
         G_STRINGIFY (NTSC_HEIGHT) ", " G_STRINGIFY (PAL_HEIGHT)
@@ -138,13 +149,18 @@ gst_dvdec_quality_get_type (void)
   static GType qtype = 0;
 
   if (qtype == 0) {
-    static const GFlagsValue values[] = {
-      {DV_QUALITY_COLOR, "DV_QUALITY_COLOR", "Color or monochrome decoding"},
-      {DV_QUALITY_AC_1, "DV_QUALITY_AC_1", "AC 1 something"},
-      {DV_QUALITY_AC_2, "DV_QUALITY_AC_2", "AC 2 something"},
-      {0, NULL, NULL}
+    static const GEnumValue values[] = {
+      {0, "DV_QUALITY_FASTEST", "Fastest decoding, low-quality mono"},
+      {1, "DV_QUALITY_AC_1", "Mono decoding using the first AC coefficient"},
+      {2, "DV_QUALITY_AC_2", "Highest quality mono decoding"},
+      {3, "DV_QUALITY_DC|DV_QUALITY_COLOUR", "Fastest colour decoding"},
+      {4, "DV_QUALITY_AC_1|DV_QUALITY_COLOUR",
+          "Colour, using only the first AC coefficient"},
+      {5, "DV_QUALITY_BEST", "Highest quality colour decoding"},
+      {0, NULL, NULL},
     };
-    qtype = g_flags_register_static ("GstDVDecQualityFlags", values);
+
+    qtype = g_enum_register_static ("GstDVDecQualityEnum", values);
   }
   return qtype;
 }
@@ -211,6 +227,7 @@ gst_dvdec_get_type (void)
       0,
       (GInstanceInitFunc) gst_dvdec_init,
     };
+
     dvdec_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstDVDec", &dvdec_info, 0);
   }
@@ -266,8 +283,8 @@ gst_dvdec_class_init (GstDVDecClass * klass)
       g_param_spec_boolean ("clamp_chroma", "Clamp chroma", "Clamp chroma",
           FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_QUALITY,
-      g_param_spec_flags ("quality", "Quality", "Decoding quality",
-          GST_TYPE_DVDEC_QUALITY, DV_QUALITY_BEST, G_PARAM_READWRITE));
+      g_param_spec_enum ("quality", "Quality", "Decoding quality",
+          GST_TYPE_DVDEC_QUALITY, DV_QUALITY_DEFAULT, G_PARAM_READWRITE));
 
   gobject_class->set_property = gst_dvdec_set_property;
   gobject_class->get_property = gst_dvdec_get_property;
@@ -291,12 +308,12 @@ gst_dvdec_init (GstDVDec * dvdec)
   dvdec->sinkpad =
       gst_pad_new_from_template (gst_static_pad_template_get (&sink_temp),
       "sink");
-  gst_element_add_pad (GST_ELEMENT (dvdec), dvdec->sinkpad);
   gst_pad_set_query_function (dvdec->sinkpad, NULL);
   gst_pad_set_convert_function (dvdec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_dvdec_sink_convert));
   gst_pad_set_formats_function (dvdec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_dvdec_get_formats));
+  gst_element_add_pad (GST_ELEMENT (dvdec), dvdec->sinkpad);
 
   dvdec->videosrcpad =
       gst_pad_new_from_template (gst_static_pad_template_get (&video_src_temp),
@@ -339,10 +356,12 @@ gst_dvdec_init (GstDVDec * dvdec)
 
   gst_element_set_loop_function (GST_ELEMENT (dvdec), gst_dvdec_loop);
 
+  dvdec->bs = NULL;
   dvdec->length = 0;
   dvdec->next_ts = 0LL;
   dvdec->end_position = -1LL;
   dvdec->need_discont = FALSE;
+  dvdec->new_media = FALSE;
   dvdec->framerate = 0;
   dvdec->height = 0;
   dvdec->frequency = 0;
@@ -371,6 +390,7 @@ gst_dvdec_get_formats (GstPad * pad)
   static const GstFormat sink_formats[] = {
     GST_FORMAT_BYTES,
     GST_FORMAT_TIME,
+    GST_FORMAT_DEFAULT,
     0
   };
 
@@ -383,10 +403,8 @@ gst_dvdec_src_convert (GstPad * pad, GstFormat src_format, gint64 src_value,
 {
   gboolean res = TRUE;
   GstDVDec *dvdec;
-  gint scale = 1;
 
   dvdec = GST_DVDEC (gst_pad_get_parent (pad));
-
   if (dvdec->length == 0)
     return FALSE;
 
@@ -396,7 +414,20 @@ gst_dvdec_src_convert (GstPad * pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
+        case GST_FORMAT_BYTES:
+          *dest_value = src_value;
+          break;
+        case GST_FORMAT_DEFAULT:
         case GST_FORMAT_TIME:
+          *dest_format = GST_FORMAT_TIME;
+          if (pad == dvdec->videosrcpad)
+            *dest_value = src_value * GST_SECOND /
+                (720 * dvdec->height * dvdec->bpp * dvdec->framerate /
+                GST_SECOND);
+          else if (pad == dvdec->audiosrcpad)
+            *dest_value = src_value * GST_SECOND /
+                (2 * dvdec->frequency * dvdec->channels);
+          break;
         default:
           res = FALSE;
       }
@@ -405,17 +436,16 @@ gst_dvdec_src_convert (GstPad * pad, GstFormat src_format, gint64 src_value,
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
           if (pad == dvdec->videosrcpad)
-            scale = 720 * dvdec->height * dvdec->bpp;
+            *dest_value = src_value * 720 * dvdec->height * dvdec->bpp *
+                dvdec->framerate / GST_SECOND;
           else if (pad == dvdec->audiosrcpad)
-            scale = dvdec->decoder->audio->num_channels * 2;
-          /* fallthrough */
+            *dest_value = 2 * src_value * dvdec->frequency *
+                dvdec->channels / GST_SECOND;
+          break;
+        case GST_FORMAT_TIME:
         case GST_FORMAT_DEFAULT:
-          if (pad == dvdec->videosrcpad)
-            *dest_value = src_value * dvdec->framerate * scale / GST_SECOND;
-          else if (pad == dvdec->audiosrcpad)
-            *dest_value =
-                src_value * dvdec->decoder->audio->frequency * scale /
-                GST_SECOND;
+          *dest_format = GST_FORMAT_TIME;
+          *dest_value = src_value;
           break;
         default:
           res = FALSE;
@@ -439,6 +469,9 @@ gst_dvdec_sink_convert (GstPad * pad, GstFormat src_format, gint64 src_value,
   if (dvdec->length == 0)
     return FALSE;
 
+  if (*dest_format == GST_FORMAT_DEFAULT)
+    *dest_format = GST_FORMAT_TIME;
+
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
@@ -452,6 +485,9 @@ gst_dvdec_sink_convert (GstPad * pad, GstFormat src_format, gint64 src_value,
           *dest_value = (frame * GST_SECOND) / dvdec->framerate;
           break;
         }
+        case GST_FORMAT_BYTES:
+          *dest_value = src_value;
+          break;
         default:
           res = FALSE;
       }
@@ -468,6 +504,9 @@ gst_dvdec_sink_convert (GstPad * pad, GstFormat src_format, gint64 src_value,
           *dest_value = frame * dvdec->length;
           break;
         }
+        case GST_FORMAT_TIME:
+          *dest_value = src_value;
+          break;
         default:
           res = FALSE;
       }
@@ -486,6 +525,7 @@ gst_dvdec_get_src_query_types (GstPad * pad)
     GST_QUERY_POSITION,
     0
   };
+
   return src_query_types;
 }
 
@@ -506,6 +546,9 @@ gst_dvdec_src_query (GstPad * pad, GstQueryType type,
           guint64 len;
           GstFormat tmp_format;
 
+          if (!dvdec->bs)
+            return FALSE;
+
           len = gst_bytestream_length (dvdec->bs);
           tmp_format = GST_FORMAT_TIME;
           if (len == -1 || !gst_pad_convert (dvdec->sinkpad,
@@ -565,7 +608,20 @@ gst_dvdec_handle_sink_event (GstDVDec * dvdec)
 
   switch (type) {
     case GST_EVENT_FLUSH:
+    case GST_EVENT_EOS:
+    case GST_EVENT_FILLER:
+    {
+      /* Forward the event to output sinks */
+      if (GST_PAD_IS_LINKED (dvdec->videosrcpad)) {
+        gst_event_ref (event);
+        gst_pad_push (dvdec->videosrcpad, GST_DATA (event));
+      }
+      if (GST_PAD_IS_LINKED (dvdec->audiosrcpad)) {
+        gst_event_ref (event);
+        gst_pad_push (dvdec->audiosrcpad, GST_DATA (event));
+      }
       break;
+    }
     case GST_EVENT_DISCONTINUOUS:
     {
       gint i;
@@ -709,16 +765,14 @@ gst_dvdec_video_link (GstPad * pad, const GstCaps * caps)
 
   structure = gst_caps_get_structure (caps, 0);
 
-  /* it worked, try to find what it was again */
-  if (!gst_structure_get_fourcc (structure, "format", &fourcc) ||
-      !gst_structure_get_int (structure, "height", &height) ||
+  if (!gst_structure_get_int (structure, "height", &height) ||
       !gst_structure_get_double (structure, "framerate", &framerate))
     return GST_PAD_LINK_REFUSED;
 
   if ((height != dvdec->height) || (framerate != dvdec->framerate))
     return GST_PAD_LINK_REFUSED;
 
-  if (fourcc == GST_STR_FOURCC ("RGB ")) {
+  if (strcmp (gst_structure_get_name (structure), "video/x-raw-rgb") == 0) {
     gint bpp;
 
     gst_structure_get_int (structure, "bpp", &bpp);
@@ -730,6 +784,9 @@ gst_dvdec_video_link (GstPad * pad, const GstCaps * caps)
       dvdec->bpp = 4;
     }
   } else {
+    if (!gst_structure_get_fourcc (structure, "format", &fourcc))
+      return GST_PAD_LINK_REFUSED;
+
     dvdec->space = e_dv_color_yuv;
     dvdec->bpp = 2;
   }
@@ -743,10 +800,12 @@ gst_dvdec_push (GstDVDec * dvdec, GstBuffer * outbuf, GstPad * pad,
 {
   GST_BUFFER_TIMESTAMP (outbuf) = ts;
 
-  if (dvdec->need_discont) {
+  if ((dvdec->need_discont) || (dvdec->new_media)) {
     GstEvent *discont;
 
     discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, ts, NULL);
+    GST_EVENT_DISCONT_NEW_MEDIA (discont) = dvdec->new_media;
+
     gst_pad_push (pad, GST_DATA (discont));
   }
 
@@ -768,15 +827,23 @@ gst_dvdec_loop (GstElement * element)
   guint8 *inframe;
   gint height;
   guint32 length, got_bytes;
-  GstFormat format;
   guint64 ts;
   gdouble fps;
 
   dvdec = GST_DVDEC (element);
 
+  /*
+   * Apparently dv_parse_header can read from the body of the frame
+   * too, so it needs more than header_size bytes. Wacky!
+   */
+  if (dvdec->found_header)
+    length = (dvdec->PAL ? PAL_BUFFER : NTSC_BUFFER);
+  else
+    length = NTSC_BUFFER;
+
   /* first read enough bytes to parse the header */
-  got_bytes = gst_bytestream_peek_bytes (dvdec->bs, &inframe, header_size);
-  if (got_bytes < header_size) {
+  got_bytes = gst_bytestream_peek_bytes (dvdec->bs, &inframe, length);
+  if (got_bytes < length) {
     gst_dvdec_handle_sink_event (dvdec);
     return;
   }
@@ -793,16 +860,6 @@ gst_dvdec_loop (GstElement * element)
   height = (dvdec->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
   length = (dvdec->PAL ? PAL_BUFFER : NTSC_BUFFER);
 
-  if ((dvdec->framerate != fps) || (dvdec->height != height)) {
-    dvdec->height = height;
-    dvdec->framerate = fps;
-
-    if (GST_PAD_LINK_FAILED (gst_pad_renegotiate (dvdec->videosrcpad))) {
-      GST_ELEMENT_ERROR (dvdec, CORE, NEGOTIATION, (NULL), (NULL));
-      return;
-    }
-  }
-
   if (length != dvdec->length) {
     dvdec->length = length;
     gst_bytestream_size_hint (dvdec->bs, length);
@@ -815,13 +872,12 @@ gst_dvdec_loop (GstElement * element)
     return;
   }
 
-  format = GST_FORMAT_TIME;
-  gst_pad_query (dvdec->videosrcpad, GST_QUERY_POSITION, &format, &ts);
-
+  ts = dvdec->next_ts;
   dvdec->next_ts += GST_SECOND / dvdec->framerate;
 
   dv_parse_packs (dvdec->decoder, GST_BUFFER_DATA (buf));
-
+  if (dv_is_new_recording (dvdec->decoder, GST_BUFFER_DATA (buf)))
+    dvdec->new_media = TRUE;
   if (GST_PAD_IS_LINKED (dvdec->audiosrcpad)) {
     gint16 *a_ptr;
     gint i, j;
@@ -829,38 +885,44 @@ gst_dvdec_loop (GstElement * element)
     dv_decode_full_audio (dvdec->decoder, GST_BUFFER_DATA (buf),
         dvdec->audio_buffers);
 
-    if ((dvdec->decoder->audio->frequency != dvdec->frequency) ||
-        (dvdec->decoder->audio->num_channels != dvdec->channels)) {
+    if ((dv_get_frequency (dvdec->decoder) != dvdec->frequency) ||
+        (dv_get_num_channels (dvdec->decoder) != dvdec->channels)) {
       if (!gst_pad_set_explicit_caps (dvdec->audiosrcpad,
               gst_caps_new_simple ("audio/x-raw-int",
-                  "rate", G_TYPE_INT, dvdec->decoder->audio->frequency,
+                  "rate", G_TYPE_INT, dv_get_frequency (dvdec->decoder),
                   "depth", G_TYPE_INT, 16,
                   "width", G_TYPE_INT, 16,
                   "signed", G_TYPE_BOOLEAN, TRUE,
-                  "channels", G_TYPE_INT, dvdec->decoder->audio->num_channels,
+                  "channels", G_TYPE_INT, dv_get_num_channels (dvdec->decoder),
                   "endianness", G_TYPE_INT, G_LITTLE_ENDIAN, NULL))) {
         gst_buffer_unref (buf);
-        GST_ELEMENT_ERROR (dvdec, CORE, NEGOTIATION, (NULL), (NULL));
+        GST_ELEMENT_ERROR (dvdec, CORE, NEGOTIATION, (NULL),
+            ("Failed to negotiate audio parameters for the DV audio stream"));
         return;
       }
 
-      dvdec->frequency = dvdec->decoder->audio->frequency;
-      dvdec->channels = dvdec->decoder->audio->num_channels;
+      dvdec->frequency = dv_get_frequency (dvdec->decoder);
+      dvdec->channels = dv_get_num_channels (dvdec->decoder);
     }
 
-    outbuf = gst_buffer_new ();
-    GST_BUFFER_SIZE (outbuf) = dvdec->decoder->audio->samples_this_frame *
-        sizeof (gint16) * dvdec->decoder->audio->num_channels;
-    GST_BUFFER_DATA (outbuf) = g_malloc (GST_BUFFER_SIZE (outbuf));
+    if (dv_get_num_samples (dvdec->decoder) > 0) {
+      outbuf = gst_buffer_new ();
+      GST_BUFFER_SIZE (outbuf) = dv_get_num_samples (dvdec->decoder) *
+          sizeof (gint16) * dv_get_num_channels (dvdec->decoder);
+      GST_BUFFER_DATA (outbuf) = g_malloc (GST_BUFFER_SIZE (outbuf));
 
-    a_ptr = (gint16 *) GST_BUFFER_DATA (outbuf);
+      a_ptr = (gint16 *) GST_BUFFER_DATA (outbuf);
 
-    for (i = 0; i < dvdec->decoder->audio->samples_this_frame; i++) {
-      for (j = 0; j < dvdec->decoder->audio->num_channels; j++) {
-        *(a_ptr++) = dvdec->audio_buffers[j][i];
+      for (i = 0; i < dv_get_num_samples (dvdec->decoder); i++) {
+        for (j = 0; j < dv_get_num_channels (dvdec->decoder); j++) {
+          *(a_ptr++) = dvdec->audio_buffers[j][i];
+        }
       }
+      gst_dvdec_push (dvdec, outbuf, dvdec->audiosrcpad, ts);
     }
-    gst_dvdec_push (dvdec, outbuf, dvdec->audiosrcpad, ts);
+  } else {
+    dvdec->frequency = dv_get_frequency (dvdec->decoder);
+    dvdec->channels = dv_get_num_channels (dvdec->decoder);
   }
 
   if (GST_PAD_IS_LINKED (dvdec->videosrcpad)) {
@@ -868,6 +930,16 @@ gst_dvdec_loop (GstElement * element)
     guint8 *outframe_ptrs[3];
     gint outframe_pitches[3];
 
+    if ((dvdec->framerate != fps) || (dvdec->height != height)) {
+      dvdec->height = height;
+      dvdec->framerate = fps;
+
+      if (GST_PAD_LINK_FAILED (gst_pad_renegotiate (dvdec->videosrcpad))) {
+        GST_ELEMENT_ERROR (dvdec, CORE, NEGOTIATION, (NULL), (NULL));
+        return;
+      }
+    }
+
     outbuf = gst_buffer_new_and_alloc ((720 * height) * dvdec->bpp);
 
     outframe = GST_BUFFER_DATA (outbuf);
@@ -888,6 +960,9 @@ gst_dvdec_loop (GstElement * element)
         dvdec->space, outframe_ptrs, outframe_pitches);
 
     gst_dvdec_push (dvdec, outbuf, dvdec->videosrcpad, ts);
+  } else {
+    dvdec->height = height;
+    dvdec->framerate = fps;
   }
 
   if ((dvdec->end_position != -1) &&
@@ -895,9 +970,8 @@ gst_dvdec_loop (GstElement * element)
     gst_element_set_eos (GST_ELEMENT (dvdec));
   }
 
-  if (dvdec->need_discont) {
-    dvdec->need_discont = FALSE;
-  }
+  dvdec->need_discont = FALSE;
+  dvdec->new_media = FALSE;
 
   gst_buffer_unref (buf);
 }
@@ -914,7 +988,12 @@ gst_dvdec_change_state (GstElement * element)
       dvdec->bs = gst_bytestream_new (dvdec->sinkpad);
       dvdec->decoder =
           dv_decoder_new (0, dvdec->clamp_luma, dvdec->clamp_chroma);
-      dvdec->decoder->quality = dvdec->quality;
+      dvdec->decoder->quality = qualities[dvdec->quality];
+      /* 
+       * Enable this function call when libdv2 0.100 or higher is more
+       * common
+       */
+      /* dv_set_quality (dvdec->decoder, qualities [dvdec->quality]); */
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       break;
@@ -925,6 +1004,7 @@ gst_dvdec_change_state (GstElement * element)
       dvdec->decoder = NULL;
       dvdec->found_header = FALSE;
       gst_bytestream_destroy (dvdec->bs);
+      dvdec->bs = NULL;
       break;
     case GST_STATE_READY_TO_NULL:
       break;
@@ -932,9 +1012,7 @@ gst_dvdec_change_state (GstElement * element)
       break;
   }
 
-  parent_class->change_state (element);
-
-  return GST_STATE_SUCCESS;
+  return parent_class->change_state (element);
 }
 
 /* Arguments are part of the Gtk+ object system, and these functions
@@ -961,7 +1039,9 @@ gst_dvdec_set_property (GObject * object, guint prop_id, const GValue * value,
       dvdec->clamp_chroma = g_value_get_boolean (value);
       break;
     case ARG_QUALITY:
-      dvdec->quality = g_value_get_flags (value);
+      dvdec->quality = g_value_get_enum (value);
+      if ((dvdec->quality < 0) || (dvdec->quality > 5))
+        dvdec->quality = 0;
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -988,7 +1068,7 @@ gst_dvdec_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_boolean (value, dvdec->clamp_chroma);
       break;
     case ARG_QUALITY:
-      g_value_set_flags (value, dvdec->quality);
+      g_value_set_enum (value, dvdec->quality);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
