@@ -40,6 +40,13 @@ GstElementDetails flacdec_details = {
   "Wim Taymans <wim.taymans@chello.be>",
 };
 
+/* FlacDec signals and args */
+enum
+{
+  /* FILL ME */
+  LAST_SIGNAL
+};
+
 enum
 {
   ARG_0,
@@ -49,7 +56,7 @@ enum
 static void gst_flacdec_base_init (gpointer g_class);
 static void gst_flacdec_class_init (FlacDecClass * klass);
 static void gst_flacdec_init (FlacDec * flacdec);
-static void gst_flacdec_finalize (GObject * object);
+static void gst_flacdec_dispose (GObject * object);
 
 static void gst_flacdec_loop (GstElement * element);
 static GstElementStateReturn gst_flacdec_change_state (GstElement * element);
@@ -87,6 +94,8 @@ static void gst_flacdec_error_callback (const FLAC__SeekableStreamDecoder *
 
 static GstElementClass *parent_class = NULL;
 
+/*static guint gst_flacdec_signals[LAST_SIGNAL] = { 0 }; */
+
 GType
 flacdec_get_type (void)
 {
@@ -116,18 +125,19 @@ flac_caps_factory (void)
 {
   return gst_caps_new_simple ("audio/x-flac", NULL);
   /* "rate",            GST_PROPS_INT_RANGE (11025, 48000),
-   * "channels",        GST_PROPS_INT_RANGE (1, 6), */
+   * "channels",        GST_PROPS_INT_RANGE (1, 2), */
 }
 
 static GstCaps *
 raw_caps_factory (void)
 {
-  return gst_caps_from_string ("audio/x-raw-int,"
-      "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER) ", "
-      "signed = (boolean) true, "
-      "width = (int) { 8, 16, 32 }, "
-      "depth = (int) { 8, 16, 24, 32 }, "
-      "rate = (int) [ 11025, 48000 ], " "channels = (int) [ 1, 6 ]");
+  return gst_caps_new_simple ("audio/x-raw-int",
+      "endianness", G_TYPE_INT, G_BYTE_ORDER,
+      "signed", G_TYPE_BOOLEAN, TRUE,
+      "width", G_TYPE_INT, 16,
+      "depth", G_TYPE_INT, 16,
+      "rate", GST_TYPE_INT_RANGE, 11025, 48000,
+      "channels", GST_TYPE_INT_RANGE, 1, 2, NULL);
 }
 
 static void
@@ -159,8 +169,8 @@ gst_flacdec_class_init (FlacDecClass * klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
-  gobject_class->finalize = gst_flacdec_finalize;
-
+  gobject_class->dispose = gst_flacdec_dispose;
+  
   gstelement_class->change_state = gst_flacdec_change_state;
 }
 
@@ -221,7 +231,7 @@ gst_flacdec_init (FlacDec * flacdec)
 }
 
 static void
-gst_flacdec_finalize (GObject * object)
+gst_flacdec_dispose (GObject * object)
 {
   FlacDec *flacdec;
 
@@ -231,7 +241,7 @@ gst_flacdec_finalize (GObject * object)
     FLAC__seekable_stream_decoder_delete (flacdec->decoder);
   flacdec->decoder = NULL;
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
@@ -252,7 +262,7 @@ gst_flacdec_update_metadata (FlacDec * flacdec,
   value = NULL;
   GST_DEBUG ("%d tag(s) found", number_of_comments);
   for (cursor = 0; cursor < number_of_comments; cursor++) {
-    str_ptr = (gchar *) metadata->data.vorbis_comment.comments[cursor].entry;
+    str_ptr = metadata->data.vorbis_comment.comments[cursor].entry;
     str_len = metadata->data.vorbis_comment.comments[cursor].length;
     p_value = g_strstr_len (str_ptr, str_len, "=");
     if (p_value) {
@@ -265,8 +275,7 @@ gst_flacdec_update_metadata (FlacDec * flacdec,
       g_free (value);
     }
   }
-  gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
-      GST_TAG_AUDIO_CODEC, "FLAC", NULL);
+
 
   gst_element_found_tags (GST_ELEMENT (flacdec), list);
   if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
@@ -445,7 +454,6 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder * decoder,
   FlacDec *flacdec;
   GstBuffer *outbuf;
   guint depth = frame->header.bits_per_sample;
-  guint width = (depth == 24) ? 32 : depth;
   guint channels = frame->header.channels;
   guint samples = frame->header.blocksize;
   guint j, i;
@@ -459,24 +467,26 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder * decoder,
 
     flacdec->need_discont = FALSE;
 
-    if (flacdec->seek_pending) {
-      flacdec->total_samples = flacdec->seek_value;
-    }
+    if (!GST_PAD_CAPS (flacdec->srcpad)) {
+      if (flacdec->seek_pending) {
+        flacdec->total_samples = flacdec->seek_value;
+      }
 
-    if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
-      GST_DEBUG ("send discont");
+      if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
+        GST_DEBUG ("send discont");
 
-      format = GST_FORMAT_TIME;
-      gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
-          flacdec->total_samples, &format, &time);
-      format = GST_FORMAT_BYTES;
-      gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
-          flacdec->total_samples, &format, &bytes);
-      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
-          GST_FORMAT_BYTES, bytes,
-          GST_FORMAT_DEFAULT, flacdec->total_samples, NULL);
+        format = GST_FORMAT_TIME;
+        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+            flacdec->total_samples, &format, &time);
+        format = GST_FORMAT_BYTES;
+        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+            flacdec->total_samples, &format, &bytes);
+        discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
+            GST_FORMAT_BYTES, bytes,
+            GST_FORMAT_DEFAULT, flacdec->total_samples, NULL);
 
-      gst_pad_push (flacdec->srcpad, GST_DATA (discont));
+        gst_pad_push (flacdec->srcpad, GST_DATA (discont));
+      }
     }
   }
 
@@ -485,12 +495,11 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder * decoder,
         gst_caps_new_simple ("audio/x-raw-int",
             "endianness", G_TYPE_INT, G_BYTE_ORDER,
             "signed", G_TYPE_BOOLEAN, TRUE,
-            "width", G_TYPE_INT, width,
+            "width", G_TYPE_INT, depth,
             "depth", G_TYPE_INT, depth,
             "rate", G_TYPE_INT, frame->header.sample_rate,
             "channels", G_TYPE_INT, channels, NULL));
 
-    flacdec->width = width;
     flacdec->depth = depth;
     flacdec->channels = channels;
     flacdec->frequency = frame->header.sample_rate;
@@ -498,12 +507,10 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder * decoder,
 
   if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
     outbuf = gst_buffer_new ();
-    GST_BUFFER_SIZE (outbuf) = samples * channels * ((width + 7) >> 3);
+    GST_BUFFER_SIZE (outbuf) = samples * channels * ((depth + 7) >> 3);
     GST_BUFFER_DATA (outbuf) = g_malloc (GST_BUFFER_SIZE (outbuf));
     GST_BUFFER_TIMESTAMP (outbuf) =
         flacdec->total_samples * GST_SECOND / frame->header.sample_rate;
-    GST_BUFFER_DURATION (outbuf) =
-        samples * GST_SECOND / frame->header.sample_rate;
 
     if (depth == 8) {
       guint8 *outbuffer = (guint8 *) GST_BUFFER_DATA (outbuf);
@@ -521,14 +528,6 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder * decoder,
           *outbuffer++ = (guint16) buffer[j][i];
         }
       }
-    } else if (depth == 24 || depth == 32) {
-      guint32 *outbuffer = (guint32 *) GST_BUFFER_DATA (outbuf);
-
-      for (i = 0; i < samples; i++) {
-        for (j = 0; j < channels; j++) {
-          *outbuffer++ = (guint32) buffer[j][i];
-        }
-      }
     } else {
       g_warning ("flacdec: invalid depth %d found\n", depth);
       return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
@@ -618,7 +617,7 @@ gst_flacdec_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
   guint scale = 1;
   gint bytes_per_sample;
 
-  bytes_per_sample = flacdec->channels * ((flacdec->width + 7) >> 3);
+  bytes_per_sample = flacdec->channels * ((flacdec->depth + 7) >> 3);
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
@@ -701,12 +700,12 @@ gst_flacdec_src_query (GstPad * pad, GstQueryType type,
       else
         samples = flacdec->stream_samples;
 
-      res = gst_pad_convert (flacdec->srcpad,
+      gst_pad_convert (flacdec->srcpad,
           GST_FORMAT_DEFAULT, samples, format, value);
       break;
     }
     case GST_QUERY_POSITION:
-      res = gst_pad_convert (flacdec->srcpad,
+      gst_pad_convert (flacdec->srcpad,
           GST_FORMAT_DEFAULT, flacdec->total_samples, format, value);
       break;
     default:
