@@ -63,6 +63,8 @@ struct _GstJpegEnc {
   gint bufsize;
   GstBuffer *buffer;
   guint row_stride;
+  /* the jpeg line buffer */
+  guchar **line[3];
 
   struct jpeg_compress_struct cinfo;
   struct jpeg_error_mgr jerr;
