@@ -30,16 +30,13 @@
 #include <gst/tag/tag.h>
 #include "vorbisenc.h"
 
-GST_DEBUG_CATEGORY_EXTERN (vorbisenc_debug);
-#define GST_CAT_DEFAULT vorbisenc_debug
-
 static GstPadTemplate *gst_vorbisenc_src_template, *gst_vorbisenc_sink_template;
 
 /* elementfactory information */
 GstElementDetails vorbisenc_details = {
-  "Vorbis encoder",
+  "Ogg Vorbis encoder",
   "Codec/Encoder/Audio",
-  "Encodes audio in Vorbis format",
+  "Encodes audio in OGG Vorbis format",
   "Monty <monty@xiph.org>, " "Wim Taymans <wim@fluendo.com>",
 };
 
@@ -96,8 +93,6 @@ gst_vorbisenc_get_formats (GstPad * pad)
 #define BITRATE_DEFAULT 	-1
 #define MIN_BITRATE_DEFAULT 	-1
 #define QUALITY_DEFAULT 	0.3
-#define LOWEST_BITRATE 		6000    /* lowest allowed for a 8 kHz stream */
-#define HIGHEST_BITRATE 	250001  /* highest allowed for a 44 kHz stream */
 
 static void gst_vorbisenc_base_init (gpointer g_class);
 static void gst_vorbisenc_class_init (VorbisEncClass * klass);
@@ -158,14 +153,12 @@ vorbis_caps_factory (void)
 static GstCaps *
 raw_caps_factory (void)
 {
-  /* lowest sample rate is in vorbis/lib/modes/setup_8.h, 8000 Hz
-   * highest sample rate is in vorbis/lib/modes/setup_44.h, 50000 Hz */
   return
       gst_caps_new_simple ("audio/x-raw-float",
-      "rate", GST_TYPE_INT_RANGE, 8000, 50000,
-      "channels", GST_TYPE_INT_RANGE, 1, 2,
       "endianness", G_TYPE_INT, G_BYTE_ORDER,
-      "width", G_TYPE_INT, 32, "buffer-frames", G_TYPE_INT, 0, NULL);
+      "width", G_TYPE_INT, 32,
+      "rate", GST_TYPE_INT_RANGE, 11025, 48000,
+      "channels", GST_TYPE_INT_RANGE, 1, 2, NULL);
 }
 
 static void
@@ -198,21 +191,17 @@ gst_vorbisenc_class_init (VorbisEncClass * klass)
   gstelement_class = (GstElementClass *) klass;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAX_BITRATE,
-      g_param_spec_int ("max-bitrate", "Maximum Bitrate",
-          "Specify a maximum bitrate (in bps). Useful for streaming "
-          "applications. (-1 == disabled)",
-          -1, HIGHEST_BITRATE, MAX_BITRATE_DEFAULT, G_PARAM_READWRITE));
+      g_param_spec_int ("max_bitrate", "Max bitrate",
+          " Specify a minimum bitrate (in bps). Useful for encoding for a fixed-size channel",
+          -1, G_MAXINT, MAX_BITRATE_DEFAULT, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BITRATE,
-      g_param_spec_int ("bitrate", "Target Bitrate",
-          "Attempt to encode at a bitrate averaging this (in bps). "
-          "This uses the bitrate management engine, and is not recommended for most users. "
-          "Quality is a better alternative. (-1 == disabled)",
-          -1, HIGHEST_BITRATE, BITRATE_DEFAULT, G_PARAM_READWRITE));
+      g_param_spec_int ("bitrate", "Bitrate", "Choose a bitrate to encode at. "
+          "Attempt to encode at a bitrate averaging this. Takes an argument in kbps.",
+          -1, G_MAXINT, BITRATE_DEFAULT, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MIN_BITRATE,
-      g_param_spec_int ("min_bitrate", "Minimum Bitrate",
-          "Specify a minimum bitrate (in bps). Useful for encoding for a "
-          "fixed-size channel. (-1 == disabled)",
-          -1, HIGHEST_BITRATE, MIN_BITRATE_DEFAULT, G_PARAM_READWRITE));
+      g_param_spec_int ("min_bitrate", "Min bitrate",
+          "Specify a maximum bitrate in bps. Useful for streaming applications.",
+          -1, G_MAXINT, MIN_BITRATE_DEFAULT, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_QUALITY,
       g_param_spec_float ("quality", "Quality",
           "Specify quality instead of specifying a particular bitrate.",
@@ -483,8 +472,6 @@ gst_vorbisenc_init (VorbisEnc * vorbisenc)
 
   vorbisenc->tags = gst_tag_list_new ();
 
-  vorbisenc->initial_delay = 0;
-
   /* we're chained and we can deal with events */
   GST_FLAG_SET (vorbisenc, GST_ELEMENT_EVENT_AWARE);
 }
@@ -507,24 +494,21 @@ gst_vorbisenc_get_tag_value (const GstTagList * list, const gchar * tag,
       || (strcmp (tag, GST_TAG_ALBUM_VOLUME_COUNT) == 0)) {
     guint track_no;
 
-    if (!gst_tag_list_get_uint_index (list, tag, index, &track_no))
-      g_assert_not_reached ();
+    g_assert (gst_tag_list_get_uint_index (list, tag, index, &track_no));
     vorbisvalue = g_strdup_printf ("%u", track_no);
   } else if (strcmp (tag, GST_TAG_DATE) == 0) {
     /* FIXME: how are dates represented in vorbis files? */
     GDate *date;
     guint u;
 
-    if (!gst_tag_list_get_uint_index (list, tag, index, &u))
-      g_assert_not_reached ();
+    g_assert (gst_tag_list_get_uint_index (list, tag, index, &u));
     date = g_date_new_julian (u);
     vorbisvalue =
         g_strdup_printf ("%04d-%02d-%02d", (gint) g_date_get_year (date),
         (gint) g_date_get_month (date), (gint) g_date_get_day (date));
     g_date_free (date);
   } else if (gst_tag_get_type (tag) == G_TYPE_STRING) {
-    if (!gst_tag_list_get_string_index (list, tag, index, &vorbisvalue))
-      g_assert_not_reached ();
+    g_assert (gst_tag_list_get_string_index (list, tag, index, &vorbisvalue));
   }
 
   return vorbisvalue;
@@ -668,8 +652,7 @@ gst_vorbisenc_setup (VorbisEnc * vorbisenc)
 
       vorbis_encode_ctl (&vorbisenc->vi, OV_ECTL_RATEMANAGE_GET, &ai);
 
-      /* the bitrates used by libvorbisenc are in kbit/sec, ours in bit/sec
-       * also remember that in telecom kbit/sec is 1000 bit/sec */
+      /* the bitrates are in kHz */
       ai.bitrate_hard_min = vorbisenc->min_bitrate / 1000;
       ai.bitrate_hard_max = vorbisenc->max_bitrate / 1000;
       ai.management_active = 1;
@@ -677,20 +660,14 @@ gst_vorbisenc_setup (VorbisEnc * vorbisenc)
       vorbis_encode_ctl (&vorbisenc->vi, OV_ECTL_RATEMANAGE_SET, &ai);
     }
   } else {
-    long min_bitrate, max_bitrate;
-
-    min_bitrate = vorbisenc->min_bitrate > 0 ? vorbisenc->min_bitrate : -1;
-    max_bitrate = vorbisenc->max_bitrate > 0 ? vorbisenc->max_bitrate : -1;
-
     if (vorbis_encode_setup_managed (&vorbisenc->vi,
             vorbisenc->channels,
             vorbisenc->frequency,
-            max_bitrate, vorbisenc->bitrate, min_bitrate) != 0) {
+            vorbisenc->max_bitrate > 0 ? vorbisenc->max_bitrate : -1,
+            vorbisenc->bitrate,
+            vorbisenc->min_bitrate > 0 ? vorbisenc->min_bitrate : -1)) {
       GST_ERROR_OBJECT (vorbisenc,
-          "vorbis_encode_setup_managed "
-          "(c %d, rate %d, max br %ld, br %ld, min br %ld) failed",
-          vorbisenc->channels, vorbisenc->frequency, max_bitrate,
-          vorbisenc->bitrate, min_bitrate);
+          "vorbisenc: initialisation failed: invalid parameters for bitrate");
       vorbis_info_clear (&vorbisenc->vi);
       return FALSE;
     }
@@ -713,49 +690,21 @@ gst_vorbisenc_setup (VorbisEnc * vorbisenc)
   return TRUE;
 }
 
-/* convert an ogg packet to a GstBuffer */
+/* prepare a buffer for transmission by passing data through libvorbis */
 static GstBuffer *
-gst_vorbisenc_buffer_from_packet (VorbisEnc * vorbisenc, ogg_packet * packet,
-    gboolean isheader)
+gst_vorbisenc_buffer_from_packet (VorbisEnc * vorbisenc, ogg_packet * packet)
 {
   GstBuffer *outbuf;
-  guint64 granulepos_delta, timestamp_delta;
-
-  GST_DEBUG_OBJECT (vorbisenc,
-      "converting ogg packet with packetno %" G_GINT64_FORMAT
-      " and granulepos %" G_GINT64_FORMAT " to GstBuffer",
-      packet->packetno, packet->granulepos);
-  /* header packets always need to have granulepos 0,
-   * regardless of the initial delay */
-  if (isheader) {
-    GST_DEBUG_OBJECT (vorbisenc, "header packet, not adjusting granulepos");
-    granulepos_delta = 0;
-    timestamp_delta = 0;
-  } else {
-    granulepos_delta =
-        vorbisenc->initial_delay * vorbisenc->frequency / GST_SECOND;
-    timestamp_delta = vorbisenc->initial_delay;
-  }
 
   outbuf = gst_buffer_new_and_alloc (packet->bytes);
   memcpy (GST_BUFFER_DATA (outbuf), packet->packet, packet->bytes);
   GST_BUFFER_OFFSET (outbuf) = vorbisenc->bytes_out;
-
-  /* hack: for all codecs used by Ogg, we set the granulepos as
-   * OFFSET_END so the muxer can mux based on granulepos, as required */
-  GST_BUFFER_OFFSET_END (outbuf) = packet->granulepos + granulepos_delta;
+  GST_BUFFER_OFFSET_END (outbuf) = packet->granulepos;
   GST_BUFFER_TIMESTAMP (outbuf) =
       vorbis_granule_time_copy (&vorbisenc->vd,
-      packet->granulepos) * GST_SECOND + timestamp_delta;
-
-  GST_DEBUG ("encoded buffer of %d bytes. timestamp = %" GST_TIME_FORMAT
-      " granulepos = %" G_GINT64_FORMAT
-      " + delta %" G_GINT64_FORMAT " = %" G_GINT64_FORMAT,
-      GST_BUFFER_SIZE (outbuf),
-      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
-      packet->granulepos, granulepos_delta,
-      packet->granulepos + granulepos_delta);
+      packet->granulepos) * GST_SECOND;
 
+  GST_DEBUG ("encoded buffer of %d bytes", GST_BUFFER_SIZE (outbuf));
   return outbuf;
 }
 
@@ -772,13 +721,12 @@ gst_vorbisenc_push_buffer (VorbisEnc * vorbisenc, GstBuffer * buffer)
   }
 }
 
-/* push a regular non-header ogg packet out as a GstBuffer */
 static void
 gst_vorbisenc_push_packet (VorbisEnc * vorbisenc, ogg_packet * packet)
 {
   GstBuffer *outbuf;
 
-  outbuf = gst_vorbisenc_buffer_from_packet (vorbisenc, packet, FALSE);
+  outbuf = gst_vorbisenc_buffer_from_packet (vorbisenc, packet);
   gst_vorbisenc_push_buffer (vorbisenc, outbuf);
 }
 
@@ -847,24 +795,6 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
         }
         gst_pad_event_default (pad, event);
         return;
-      case GST_EVENT_DISCONTINUOUS:
-      {
-        gint64 val;
-
-        if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &val)) {
-          /* vorbis does not support discontinuities in the middle of
-           * a stream so we can't just increase the granulepos to reflect
-           * the new position, or can we? would that still be according
-           * to spec? */
-          if (vorbisenc->samples_in == 0) {
-            vorbisenc->initial_delay = val;
-            GST_DEBUG ("initial delay = %" G_GUINT64_FORMAT, val);
-          } else {
-            GST_DEBUG ("mid stream discont: val = %" G_GUINT64_FORMAT, val);
-          }
-        }
-      }
-        /* fall through */
       default:
         gst_pad_event_default (pad, event);
         return;
@@ -902,9 +832,9 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
           &header_comm, &header_code);
 
       /* create header buffers */
-      buf1 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header, TRUE);
-      buf2 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_comm, TRUE);
-      buf3 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_code, TRUE);
+      buf1 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header);
+      buf2 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_comm);
+      buf3 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_code);
 
       /* mark and put on caps */
       caps = gst_pad_get_caps (vorbisenc->srcpad);
@@ -1022,11 +952,6 @@ gst_vorbisenc_set_property (GObject * object, guint prop_id,
       gboolean old_value = vorbisenc->managed;
 
       vorbisenc->max_bitrate = g_value_get_int (value);
-      if (vorbisenc->max_bitrate >= 0
-          && vorbisenc->max_bitrate < LOWEST_BITRATE) {
-        g_warning ("Lowest allowed bitrate is %d", LOWEST_BITRATE);
-        vorbisenc->max_bitrate = LOWEST_BITRATE;
-      }
       if (vorbisenc->min_bitrate > 0 && vorbisenc->max_bitrate > 0)
         vorbisenc->managed = TRUE;
       else
@@ -1038,21 +963,12 @@ gst_vorbisenc_set_property (GObject * object, guint prop_id,
     }
     case ARG_BITRATE:
       vorbisenc->bitrate = g_value_get_int (value);
-      if (vorbisenc->bitrate >= 0 && vorbisenc->bitrate < LOWEST_BITRATE) {
-        g_warning ("Lowest allowed bitrate is %d", LOWEST_BITRATE);
-        vorbisenc->bitrate = LOWEST_BITRATE;
-      }
       break;
     case ARG_MIN_BITRATE:
     {
       gboolean old_value = vorbisenc->managed;
 
       vorbisenc->min_bitrate = g_value_get_int (value);
-      if (vorbisenc->min_bitrate >= 0
-          && vorbisenc->min_bitrate < LOWEST_BITRATE) {
-        g_warning ("Lowest allowed bitrate is %d", LOWEST_BITRATE);
-        vorbisenc->min_bitrate = LOWEST_BITRATE;
-      }
       if (vorbisenc->min_bitrate > 0 && vorbisenc->max_bitrate > 0)
         vorbisenc->managed = TRUE;
       else
@@ -1094,11 +1010,6 @@ gst_vorbisenc_change_state (GstElement * element)
     case GST_STATE_PAUSED_TO_READY:
       vorbisenc->setup = FALSE;
       vorbisenc->header_sent = FALSE;
-      vorbisenc->initial_delay = 0;
-      vorbisenc->samples_in = 0;
-      vorbisenc->bytes_out = 0;
-      vorbisenc->channels = -1;
-      vorbisenc->frequency = -1;
       gst_tag_list_free (vorbisenc->tags);
       vorbisenc->tags = gst_tag_list_new ();
       break;
