@@ -149,46 +149,47 @@ deep_notify_callback (GObject * object, GstObject * origin,
 
 typedef struct
 {
-  GstTagList *streaminfo;
-  GstTagList *metadata;
+  guint meta;
+  guint encoded;
 }
-TagListDest;
+TagFlagScore;
 
 static void
-tag_list_dest (const GstTagList * list, const gchar * tag, gpointer user_data)
+tag_flag_score (const GstTagList * list, const gchar * tag, gpointer user_data)
 {
-  TagListDest *dest = (TagListDest *) user_data;
+  TagFlagScore *score = (TagFlagScore *) user_data;
   GstTagFlag flag;
 
   flag = gst_tag_get_flag (tag);
+  if (flag == GST_TAG_FLAG_META)
+    score->meta++;
   if (flag == GST_TAG_FLAG_ENCODED)
-    gst_tag_list_remove_tag (dest->metadata, tag);
-  else if (flag == GST_TAG_FLAG_META)
-    gst_tag_list_remove_tag (dest->streaminfo, tag);
-  else
-    /* default to streaminfo ? */
-    gst_tag_list_remove_tag (dest->metadata, tag);
+    score->encoded++;
 }
 
 void
 found_tag_callback (GObject * pipeline, GstElement * source, GstTagList * tags,
     GstMediaInfoPriv * priv)
 {
-  TagListDest dest;
+  TagFlagScore score;
 
+  score.meta = 0;
+  score.encoded = 0;
   GST_DEBUG ("element %s found tag", GST_STR_NULL (GST_ELEMENT_NAME (source)));
 
-  dest.streaminfo = gst_tag_list_copy (tags);
-  dest.metadata = gst_tag_list_copy (tags);
+  /* decide if it's likely to be metadata or streaminfo */
+  /* FIXME: this is a hack, there must be a better way,
+     but as long as elements can report both mixed we need to do this */
 
-  gst_tag_list_foreach (tags, tag_list_dest, &dest);
+  gst_tag_list_foreach (tags, tag_flag_score, &score);
 
-  priv->metadata =
-      gst_tag_list_merge (priv->metadata, dest.metadata, GST_TAG_MERGE_REPLACE);
-
-  priv->streaminfo =
-      gst_tag_list_merge (priv->streaminfo, dest.streaminfo,
-      GST_TAG_MERGE_REPLACE);
+  if (score.meta > score.encoded) {
+    GST_DEBUG ("found tags from decoder, adding them as metadata");
+    priv->metadata = gst_tag_list_copy (tags);
+  } else {
+    GST_DEBUG ("found tags, adding them as streaminfo");
+    priv->streaminfo = gst_tag_list_copy (tags);
+  }
 }
 
 void
@@ -372,7 +373,7 @@ gmi_set_mime (GstMediaInfo * info, const char *mime)
       || (strcmp (mime, "audio/x-wav") == 0))
     desc =
         g_strdup_printf
-        ("%s name=source ! wavparse name=decoder ! audio/x-raw-int ! fakesink name=sink",
+        ("%s ! wavparse name=decoder ! audio/x-raw-int ! fakesink name=sink",
         priv->source_name);
   else if (strcmp (mime, "audio/x-mod") == 0
       || strcmp (mime, "audio/x-s3m") == 0 || strcmp (mime, "audio/x-xm") == 0
