@@ -24,32 +24,33 @@
 #endif
 
 #include <string.h>
+#include <gst/gstutils.h>
 
 #include "riff-ids.h"
 #include "riff-read.h"
 
-enum {
+enum
+{
   ARG_0,
   ARG_METADATA
-  /* FILL ME */
+      /* FILL ME */
 };
 
-static void     gst_riff_read_class_init   (GstRiffReadClass *klass);
-static void     gst_riff_read_init         (GstRiffRead *riff);
+static void gst_riff_read_class_init (GstRiffReadClass * klass);
+static void gst_riff_read_init (GstRiffRead * riff);
 
-static GstElementStateReturn
-		gst_riff_read_change_state (GstElement  *element);
+static GstElementStateReturn gst_riff_read_change_state (GstElement * element);
 
 static GstElementClass *parent_class = NULL;
 
 GType
-gst_riff_read_get_type (void) 
+gst_riff_read_get_type (void)
 {
   static GType gst_riff_read_type = 0;
 
   if (!gst_riff_read_type) {
     static const GTypeInfo gst_riff_read_info = {
-      sizeof (GstRiffReadClass),      
+      sizeof (GstRiffReadClass),
       NULL,
       NULL,
       (GClassInitFunc) gst_riff_read_class_init,
@@ -61,25 +62,25 @@ gst_riff_read_get_type (void)
     };
 
     gst_riff_read_type =
-	g_type_register_static (GST_TYPE_ELEMENT, "GstRiffRead",
-				&gst_riff_read_info, 0);
+        g_type_register_static (GST_TYPE_ELEMENT, "GstRiffRead",
+        &gst_riff_read_info, 0);
   }
 
   return gst_riff_read_type;
 }
 
 static void
-gst_riff_read_class_init (GstRiffReadClass *klass) 
+gst_riff_read_class_init (GstRiffReadClass * klass)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
-  
+
   gstelement_class->change_state = gst_riff_read_change_state;
 }
 
 static void
-gst_riff_read_init (GstRiffRead *riff)
+gst_riff_read_init (GstRiffRead * riff)
 {
   riff->sinkpad = NULL;
   riff->bs = NULL;
@@ -87,7 +88,7 @@ gst_riff_read_init (GstRiffRead *riff)
 }
 
 static GstElementStateReturn
-gst_riff_read_change_state (GstElement *element)
+gst_riff_read_change_state (GstElement * element)
 {
   GstRiffRead *riff = GST_RIFF_READ (element);
 
@@ -124,7 +125,7 @@ gst_riff_read_change_state (GstElement *element)
  */
 
 static guint
-gst_riff_read_element_level_up (GstRiffRead *riff)
+gst_riff_read_element_level_up (GstRiffRead * riff)
 {
   guint num = 0;
   guint64 pos = gst_bytestream_tell (riff->bs);
@@ -149,14 +150,28 @@ gst_riff_read_element_level_up (GstRiffRead *riff)
  * TRUE on success or FALSE on failure.
  */
 
-static gboolean
-gst_riff_peek_head (GstRiffRead *riff,
-		    guint32     *tag,
-		    guint32     *length,
-		    guint       *level_up)
+gboolean
+gst_riff_peek_head (GstRiffRead * riff,
+    guint32 * tag, guint32 * length, guint * level_up)
 {
+  GList *last;
   guint8 *data;
 
+  /* if we're at the end of a chunk, but unaligned, then re-align.
+   * Those are essentially broken files, but unfortunately they
+   * exist. */
+  if ((last = g_list_last (riff->level)) != NULL) {
+    GstRiffLevel *level = last->data;
+    guint64 pos = gst_bytestream_tell (riff->bs);
+
+    if (level->start + level->length - pos < 8) {
+      if (!gst_bytestream_flush (riff->bs, level->start + level->length - pos)) {
+        GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
+        return FALSE;
+      }
+    }
+  }
+
   /* read */
   while (gst_bytestream_peek_bytes (riff->bs, &data, 8) != 8) {
     GstEvent *event = NULL;
@@ -164,19 +179,22 @@ gst_riff_peek_head (GstRiffRead *riff,
 
     /* Here, we might encounter EOS */
     gst_bytestream_get_status (riff->bs, &remaining, &event);
-    if (event) {
+    if (GST_IS_EVENT (event)) {
+      gboolean eos = (GST_EVENT_TYPE (event) == GST_EVENT_EOS);
+
       gst_pad_event_default (riff->sinkpad, event);
+      if (eos)
+        return FALSE;
     } else {
-      gst_event_unref (event);
       GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
       return FALSE;
     }
   }
 
   /* parse tag + length (if wanted) */
-  *tag = GUINT32_FROM_LE (((guint32 *) data)[0]);
+  *tag = GST_READ_UINT32_LE (data);
   if (length)
-    *length = GUINT32_FROM_LE (((guint32 *) data)[1]);
+    *length = GST_READ_UINT32_LE (((guint32 *) data) + 1);
 
   /* level */
   if (level_up)
@@ -190,17 +208,40 @@ gst_riff_peek_head (GstRiffRead *riff,
  * Return: the data, as a GstBuffer.
  */
 
-static GstBuffer *
-gst_riff_read_element_data (GstRiffRead *riff,
-			    guint        length)
+GstBuffer *
+gst_riff_read_element_data (GstRiffRead * riff, guint length, guint * got_bytes)
 {
   GstBuffer *buf = NULL;
+  guint32 got;
 
-  if (gst_bytestream_peek (riff->bs, &buf, length) != length) {
-    GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
-    if (buf)
-      gst_buffer_unref (buf);
-    return NULL;
+  while ((got = gst_bytestream_peek (riff->bs, &buf, length)) != length) {
+    /*GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL)); */
+    GstEvent *event = NULL;
+    guint32 remaining;
+
+    gst_bytestream_get_status (riff->bs, &remaining, &event);
+    if (GST_IS_EVENT (event)) {
+      gst_pad_event_default (riff->sinkpad, event);
+      if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+
+        if (buf)
+          gst_buffer_unref (buf);
+
+        if (got_bytes)
+          *got_bytes = got;
+
+        return NULL;
+      }
+    } else {
+      GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
+      if (buf)
+        gst_buffer_unref (buf);
+
+      if (got_bytes)
+        *got_bytes = got;
+
+      return NULL;
+    }
   }
 
   /* we need 16-bit alignment */
@@ -209,6 +250,9 @@ gst_riff_read_element_data (GstRiffRead *riff,
 
   gst_bytestream_flush (riff->bs, length);
 
+  if (got_bytes)
+    *got_bytes = got;
+
   return buf;
 }
 
@@ -217,8 +261,7 @@ gst_riff_read_element_data (GstRiffRead *riff,
  */
 
 GstEvent *
-gst_riff_read_seek (GstRiffRead *riff,
-		    guint64      offset)
+gst_riff_read_seek (GstRiffRead * riff, guint64 offset)
 {
   guint64 length = gst_bytestream_length (riff->bs);
   guint32 remaining;
@@ -258,7 +301,11 @@ gst_riff_read_seek (GstRiffRead *riff,
       GST_WARNING ("No discontinuity event after seek - seek failed");
       break;
     } else if (GST_EVENT_TYPE (event) != GST_EVENT_DISCONTINUOUS) {
+      GstEventType type = GST_EVENT_TYPE (event);
+
       gst_pad_event_default (riff->sinkpad, event);
+      if (type == GST_EVENT_EOS)
+        return NULL;
       event = NULL;
     }
   }
@@ -271,8 +318,7 @@ gst_riff_read_seek (GstRiffRead *riff,
  */
 
 guint32
-gst_riff_peek_tag (GstRiffRead *riff,
-		   guint       *level_up)
+gst_riff_peek_tag (GstRiffRead * riff, guint * level_up)
 {
   guint32 tag;
 
@@ -287,7 +333,7 @@ gst_riff_peek_tag (GstRiffRead *riff,
  */
 
 guint32
-gst_riff_peek_list (GstRiffRead *riff)
+gst_riff_peek_list (GstRiffRead * riff)
 {
   guint32 lst;
   guint8 *data;
@@ -304,7 +350,7 @@ gst_riff_peek_list (GstRiffRead *riff)
     return 0;
   }
 
-  return GUINT32_FROM_LE (((guint32 *) data)[2]);
+  return GST_READ_UINT32_LE (((guint32 *) data) + 2);
 }
 
 /*
@@ -312,7 +358,7 @@ gst_riff_peek_list (GstRiffRead *riff)
  */
 
 gboolean
-gst_riff_read_skip (GstRiffRead *riff)
+gst_riff_read_skip (GstRiffRead * riff)
 {
   guint32 tag, length;
   GstEvent *event;
@@ -343,7 +389,7 @@ gst_riff_read_skip (GstRiffRead *riff)
 
   /* no */
   if (!(event = gst_riff_read_seek (riff,
-			gst_bytestream_tell (riff->bs) + length)))
+              gst_bytestream_tell (riff->bs) + length)))
     return FALSE;
 
   gst_event_unref (event);
@@ -356,9 +402,7 @@ gst_riff_read_skip (GstRiffRead *riff)
  */
 
 gboolean
-gst_riff_read_data (GstRiffRead *riff,
-		    guint32     *tag,
-		    GstBuffer  **buf)
+gst_riff_read_data (GstRiffRead * riff, guint32 * tag, GstBuffer ** buf)
 {
   guint32 length;
 
@@ -366,7 +410,7 @@ gst_riff_read_data (GstRiffRead *riff,
     return FALSE;
   gst_bytestream_flush_fast (riff->bs, 8);
 
-  return ((*buf = gst_riff_read_element_data (riff, length)) != NULL);
+  return ((*buf = gst_riff_read_element_data (riff, length, NULL)) != NULL);
 }
 
 /*
@@ -374,9 +418,7 @@ gst_riff_read_data (GstRiffRead *riff,
  */
 
 gboolean
-gst_riff_read_ascii (GstRiffRead *riff,
-		     guint32     *tag,
-		     gchar      **str)
+gst_riff_read_ascii (GstRiffRead * riff, guint32 * tag, gchar ** str)
 {
   GstBuffer *buf;
 
@@ -397,8 +439,7 @@ gst_riff_read_ascii (GstRiffRead *riff,
  */
 
 gboolean
-gst_riff_read_strh (GstRiffRead    *riff,
-		    gst_riff_strh **header)
+gst_riff_read_strh (GstRiffRead * riff, gst_riff_strh ** header)
 {
   guint32 tag;
   GstBuffer *buf;
@@ -414,28 +455,27 @@ gst_riff_read_strh (GstRiffRead    *riff,
   }
   if (GST_BUFFER_SIZE (buf) < sizeof (gst_riff_strh)) {
     g_warning ("Too small strh (%d available, %d needed)",
-	       GST_BUFFER_SIZE (buf), sizeof (gst_riff_strh));
+        GST_BUFFER_SIZE (buf), (int) sizeof (gst_riff_strh));
     gst_buffer_unref (buf);
     return FALSE;
   }
 
-  strh = g_memdup (GST_BUFFER_DATA (buf),
-		   GST_BUFFER_SIZE (buf));
+  strh = g_memdup (GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
   gst_buffer_unref (buf);
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-  strh->type        = GUINT32_FROM_LE (strh->type);
+  strh->type = GUINT32_FROM_LE (strh->type);
   strh->fcc_handler = GUINT32_FROM_LE (strh->fcc_handler);
-  strh->flags       = GUINT32_FROM_LE (strh->flags);
-  strh->priority    = GUINT32_FROM_LE (strh->priority);
+  strh->flags = GUINT32_FROM_LE (strh->flags);
+  strh->priority = GUINT32_FROM_LE (strh->priority);
   strh->init_frames = GUINT32_FROM_LE (strh->init_frames);
-  strh->scale       = GUINT32_FROM_LE (strh->scale);
-  strh->rate        = GUINT32_FROM_LE (strh->rate);
-  strh->start       = GUINT32_FROM_LE (strh->start);
-  strh->length      = GUINT32_FROM_LE (strh->length);
-  strh->bufsize     = GUINT32_FROM_LE (strh->bufsize);
-  strh->quality     = GUINT32_FROM_LE (strh->quality);
-  strh->samplesize  = GUINT32_FROM_LE (strh->samplesize);
+  strh->scale = GUINT32_FROM_LE (strh->scale);
+  strh->rate = GUINT32_FROM_LE (strh->rate);
+  strh->start = GUINT32_FROM_LE (strh->start);
+  strh->length = GUINT32_FROM_LE (strh->length);
+  strh->bufsize = GUINT32_FROM_LE (strh->bufsize);
+  strh->quality = GUINT32_FROM_LE (strh->quality);
+  strh->samplesize = GUINT32_FROM_LE (strh->samplesize);
 #endif
 
   /* avoid divisions by zero */
@@ -446,20 +486,19 @@ gst_riff_read_strh (GstRiffRead    *riff,
 
   /* debug */
   GST_INFO ("strh tag found");
-  GST_INFO (" type        " GST_FOURCC_FORMAT,
-	    GST_FOURCC_ARGS (strh->type));
+  GST_INFO (" type        " GST_FOURCC_FORMAT, GST_FOURCC_ARGS (strh->type));
   GST_INFO (" fcc_handler " GST_FOURCC_FORMAT,
-	    GST_FOURCC_ARGS (strh->fcc_handler));
+      GST_FOURCC_ARGS (strh->fcc_handler));
   GST_INFO (" flags       0x%08x", strh->flags);
-  GST_INFO (" priority    %d",     strh->priority);
-  GST_INFO (" init_frames %d",     strh->init_frames);
-  GST_INFO (" scale       %d",     strh->scale);
-  GST_INFO (" rate        %d",     strh->rate);
-  GST_INFO (" start       %d",     strh->start);
-  GST_INFO (" length      %d",     strh->length);
-  GST_INFO (" bufsize     %d",     strh->bufsize);
-  GST_INFO (" quality     %d",     strh->quality);
-  GST_INFO (" samplesize  %d",     strh->samplesize);
+  GST_INFO (" priority    %d", strh->priority);
+  GST_INFO (" init_frames %d", strh->init_frames);
+  GST_INFO (" scale       %d", strh->scale);
+  GST_INFO (" rate        %d", strh->rate);
+  GST_INFO (" start       %d", strh->start);
+  GST_INFO (" length      %d", strh->length);
+  GST_INFO (" bufsize     %d", strh->bufsize);
+  GST_INFO (" quality     %d", strh->quality);
+  GST_INFO (" samplesize  %d", strh->samplesize);
 
   *header = strh;
 
@@ -467,8 +506,8 @@ gst_riff_read_strh (GstRiffRead    *riff,
 }
 
 gboolean
-gst_riff_read_strf_vids (GstRiffRead         *riff,
-			 gst_riff_strf_vids **header)
+gst_riff_read_strf_vids_with_data (GstRiffRead * riff,
+    gst_riff_strf_vids ** header, GstBuffer ** extradata)
 {
   guint32 tag;
   GstBuffer *buf;
@@ -484,33 +523,36 @@ gst_riff_read_strf_vids (GstRiffRead         *riff,
   }
   if (GST_BUFFER_SIZE (buf) < sizeof (gst_riff_strf_vids)) {
     g_warning ("Too small strf_vids (%d available, %d needed)",
-	       GST_BUFFER_SIZE (buf), sizeof (gst_riff_strf_vids));
+        GST_BUFFER_SIZE (buf), (int) sizeof (gst_riff_strf_vids));
     gst_buffer_unref (buf);
     return FALSE;
   }
 
-  strf = g_memdup (GST_BUFFER_DATA (buf),
-		   GST_BUFFER_SIZE (buf));
+  strf = g_memdup (GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-  strf->size        = GUINT32_FROM_LE (strf->size);
-  strf->width       = GUINT32_FROM_LE (strf->width);
-  strf->height      = GUINT32_FROM_LE (strf->height);
-  strf->planes      = GUINT16_FROM_LE (strf->planes);
-  strf->bit_cnt     = GUINT16_FROM_LE (strf->bit_cnt);
+  strf->size = GUINT32_FROM_LE (strf->size);
+  strf->width = GUINT32_FROM_LE (strf->width);
+  strf->height = GUINT32_FROM_LE (strf->height);
+  strf->planes = GUINT16_FROM_LE (strf->planes);
+  strf->bit_cnt = GUINT16_FROM_LE (strf->bit_cnt);
   strf->compression = GUINT32_FROM_LE (strf->compression);
-  strf->image_size  = GUINT32_FROM_LE (strf->image_size);
+  strf->image_size = GUINT32_FROM_LE (strf->image_size);
   strf->xpels_meter = GUINT32_FROM_LE (strf->xpels_meter);
   strf->ypels_meter = GUINT32_FROM_LE (strf->ypels_meter);
-  strf->num_colors  = GUINT32_FROM_LE (strf->num_colors);
-  strf->imp_colors  = GUINT32_FROM_LE (strf->imp_colors);
+  strf->num_colors = GUINT32_FROM_LE (strf->num_colors);
+  strf->imp_colors = GUINT32_FROM_LE (strf->imp_colors);
 #endif
 
   /* size checking */
+  *extradata = NULL;
   if (strf->size > GST_BUFFER_SIZE (buf)) {
     g_warning ("strf_vids header gave %d bytes data, only %d available",
-	       strf->size, GST_BUFFER_SIZE (buf));
+        strf->size, GST_BUFFER_SIZE (buf));
     strf->size = GST_BUFFER_SIZE (buf);
+  } else if (strf->size < GST_BUFFER_SIZE (buf)) {
+    *extradata = gst_buffer_create_sub (buf, strf->size,
+        GST_BUFFER_SIZE (buf) - strf->size);
   }
 
   /* debug */
@@ -521,12 +563,14 @@ gst_riff_read_strf_vids (GstRiffRead         *riff,
   GST_INFO (" planes      %d", strf->planes);
   GST_INFO (" bit_cnt     %d", strf->bit_cnt);
   GST_INFO (" compression " GST_FOURCC_FORMAT,
-	    GST_FOURCC_ARGS (strf->compression));
+      GST_FOURCC_ARGS (strf->compression));
   GST_INFO (" image_size  %d", strf->image_size);
   GST_INFO (" xpels_meter %d", strf->xpels_meter);
   GST_INFO (" ypels_meter %d", strf->ypels_meter);
   GST_INFO (" num_colors  %d", strf->num_colors);
   GST_INFO (" imp_colors  %d", strf->imp_colors);
+  if (*extradata)
+    GST_INFO (" %d bytes extra_data", GST_BUFFER_SIZE (*extradata));
 
   gst_buffer_unref (buf);
 
@@ -535,9 +579,25 @@ gst_riff_read_strf_vids (GstRiffRead         *riff,
   return TRUE;
 }
 
+/*
+ * Obsolete, use gst_riff_read_strf_vids_with_data ().
+ */
+
 gboolean
-gst_riff_read_strf_auds (GstRiffRead         *riff,
-			 gst_riff_strf_auds **header)
+gst_riff_read_strf_vids (GstRiffRead * riff, gst_riff_strf_vids ** header)
+{
+  GstBuffer *data = NULL;
+  gboolean ret;
+
+  ret = gst_riff_read_strf_vids_with_data (riff, header, &data);
+  if (data)
+    gst_buffer_unref (data);
+
+  return ret;
+}
+
+gboolean
+gst_riff_read_strf_auds (GstRiffRead * riff, gst_riff_strf_auds ** header)
 {
   guint32 tag;
   GstBuffer *buf;
@@ -546,28 +606,27 @@ gst_riff_read_strf_auds (GstRiffRead         *riff,
   if (!gst_riff_read_data (riff, &tag, &buf))
     return FALSE;
 
-  if (tag != GST_RIFF_TAG_strf) {
+  if (tag != GST_RIFF_TAG_strf && tag != GST_RIFF_TAG_fmt) {
     g_warning ("Not a strf chunk");
     gst_buffer_unref (buf);
     return FALSE;
   }
   if (GST_BUFFER_SIZE (buf) < sizeof (gst_riff_strf_auds)) {
     g_warning ("Too small strf_auds (%d available, %d needed)",
-	       GST_BUFFER_SIZE (buf), sizeof (gst_riff_strf_auds));
+        GST_BUFFER_SIZE (buf), (int) sizeof (gst_riff_strf_auds));
     gst_buffer_unref (buf);
     return FALSE;
   }
 
-  strf = g_memdup (GST_BUFFER_DATA (buf),
-		   GST_BUFFER_SIZE (buf));
+  strf = g_memdup (GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-  strf->format     = GUINT16_FROM_LE (strf->format);
-  strf->channels   = GUINT16_FROM_LE (strf->channels);
-  strf->rate       = GUINT32_FROM_LE (strf->rate);
-  strf->av_bps     = GUINT32_FROM_LE (strf->av_bps);
+  strf->format = GUINT16_FROM_LE (strf->format);
+  strf->channels = GUINT16_FROM_LE (strf->channels);
+  strf->rate = GUINT32_FROM_LE (strf->rate);
+  strf->av_bps = GUINT32_FROM_LE (strf->av_bps);
   strf->blockalign = GUINT16_FROM_LE (strf->blockalign);
-  strf->size       = GUINT16_FROM_LE (strf->size);
+  strf->size = GUINT16_FROM_LE (strf->size);
 #endif
 
   /* debug */
@@ -577,7 +636,7 @@ gst_riff_read_strf_auds (GstRiffRead         *riff,
   GST_INFO (" rate        %d", strf->rate);
   GST_INFO (" av_bps      %d", strf->av_bps);
   GST_INFO (" blockalign  %d", strf->blockalign);
-  GST_INFO (" size        %d", strf->size); /* wordsize, not extrasize! */
+  GST_INFO (" size        %d", strf->size);     /* wordsize, not extrasize! */
 
   gst_buffer_unref (buf);
 
@@ -587,8 +646,7 @@ gst_riff_read_strf_auds (GstRiffRead         *riff,
 }
 
 gboolean
-gst_riff_read_strf_iavs (GstRiffRead         *riff,
-			 gst_riff_strf_iavs **header)
+gst_riff_read_strf_iavs (GstRiffRead * riff, gst_riff_strf_iavs ** header)
 {
   guint32 tag;
   GstBuffer *buf;
@@ -604,22 +662,21 @@ gst_riff_read_strf_iavs (GstRiffRead         *riff,
   }
   if (GST_BUFFER_SIZE (buf) < sizeof (gst_riff_strf_iavs)) {
     g_warning ("Too small strf_iavs (%d available, %d needed)",
-	       GST_BUFFER_SIZE (buf), sizeof (gst_riff_strf_iavs));
+        GST_BUFFER_SIZE (buf), (int) sizeof (gst_riff_strf_iavs));
     gst_buffer_unref (buf);
     return FALSE;
   }
 
-  strf = g_memdup (GST_BUFFER_DATA (buf),
-		   GST_BUFFER_SIZE (buf));
+  strf = g_memdup (GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
   gst_buffer_unref (buf);
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-  strf->DVAAuxSrc   = GUINT32_FROM_LE (strf->DVAAuxSrc);
-  strf->DVAAuxCtl   = GUINT32_FROM_LE (strf->DVAAuxCtl);
-  strf->DVAAuxSrc1  = GUINT32_FROM_LE (strf->DVAAuxSrc1);
-  strf->DVAAuxCtl1  = GUINT32_FROM_LE (strf->DVAAuxCtl1);
-  strf->DVVAuxSrc   = GUINT32_FROM_LE (strf->DVVAuxSrc);
-  strf->DVVAuxCtl   = GUINT32_FROM_LE (strf->DVVAuxCtl);
+  strf->DVAAuxSrc = GUINT32_FROM_LE (strf->DVAAuxSrc);
+  strf->DVAAuxCtl = GUINT32_FROM_LE (strf->DVAAuxCtl);
+  strf->DVAAuxSrc1 = GUINT32_FROM_LE (strf->DVAAuxSrc1);
+  strf->DVAAuxCtl1 = GUINT32_FROM_LE (strf->DVAAuxCtl1);
+  strf->DVVAuxSrc = GUINT32_FROM_LE (strf->DVVAuxSrc);
+  strf->DVVAuxCtl = GUINT32_FROM_LE (strf->DVVAuxCtl);
   strf->DVReserved1 = GUINT32_FROM_LE (strf->DVReserved1);
   strf->DVReserved2 = GUINT32_FROM_LE (strf->DVReserved2);
 #endif
@@ -645,8 +702,7 @@ gst_riff_read_strf_iavs (GstRiffRead         *riff,
  */
 
 gboolean
-gst_riff_read_list (GstRiffRead *riff,
-		    guint32     *tag)
+gst_riff_read_list (GstRiffRead * riff, guint32 * tag)
 {
   guint32 length, lst;
   GstRiffLevel *level;
@@ -664,7 +720,7 @@ gst_riff_read_list (GstRiffRead *riff,
     return FALSE;
   }
   gst_bytestream_flush_fast (riff->bs, 4);
-  *tag = GUINT32_FROM_LE (* (guint32 *) data);
+  *tag = GST_READ_UINT32_LE (data);
 
   /* remember level */
   level = g_new (GstRiffLevel, 1);
@@ -680,7 +736,7 @@ gst_riff_read_list (GstRiffRead *riff,
  */
 
 gboolean
-gst_riff_read_info (GstRiffRead *riff)
+gst_riff_read_info (GstRiffRead * riff)
 {
   guint32 tag;
   guint64 end;
@@ -716,7 +772,7 @@ gst_riff_read_info (GstRiffRead *riff)
         type = GST_TAG_ARTIST;
         break;
       case GST_RIFF_INFO_ICMS:
-        type = NULL; /*"Commissioner";*/
+        type = NULL;            /*"Commissioner"; */
         break;
       case GST_RIFF_INFO_ICMT:
         type = GST_TAG_COMMENT;
@@ -728,60 +784,60 @@ gst_riff_read_info (GstRiffRead *riff)
         type = GST_TAG_DATE;
         break;
       case GST_RIFF_INFO_ICRP:
-        type = NULL; /*"Cropped";*/
+        type = NULL;            /*"Cropped"; */
         break;
       case GST_RIFF_INFO_IDIM:
-        type = NULL; /*"Dimensions";*/
+        type = NULL;            /*"Dimensions"; */
         break;
       case GST_RIFF_INFO_IDPI:
-        type = NULL; /*"Dots per Inch";*/
+        type = NULL;            /*"Dots per Inch"; */
         break;
       case GST_RIFF_INFO_IENG:
-        type = NULL; /*"Engineer";*/
+        type = NULL;            /*"Engineer"; */
         break;
       case GST_RIFF_INFO_IGNR:
         type = GST_TAG_GENRE;
         break;
       case GST_RIFF_INFO_IKEY:
-        type = NULL; /*"Keywords";*/;
+        type = NULL; /*"Keywords"; */ ;
         break;
       case GST_RIFF_INFO_ILGT:
-        type = NULL; /*"Lightness";*/
+        type = NULL;            /*"Lightness"; */
         break;
       case GST_RIFF_INFO_IMED:
-        type = NULL; /*"Medium";*/
+        type = NULL;            /*"Medium"; */
         break;
       case GST_RIFF_INFO_INAM:
         type = GST_TAG_TITLE;
         break;
       case GST_RIFF_INFO_IPLT:
-        type = NULL; /*"Palette";*/
+        type = NULL;            /*"Palette"; */
         break;
       case GST_RIFF_INFO_IPRD:
-        type = NULL; /*"Product";*/
+        type = NULL;            /*"Product"; */
         break;
       case GST_RIFF_INFO_ISBJ:
-        type = NULL; /*"Subject";*/
+        type = NULL;            /*"Subject"; */
         break;
       case GST_RIFF_INFO_ISFT:
         type = GST_TAG_ENCODER;
         break;
       case GST_RIFF_INFO_ISHP:
-        type = NULL; /*"Sharpness";*/
+        type = NULL;            /*"Sharpness"; */
         break;
       case GST_RIFF_INFO_ISRC:
         type = GST_TAG_ISRC;
         break;
       case GST_RIFF_INFO_ISRF:
-        type = NULL; /*"Source Form";*/
+        type = NULL;            /*"Source Form"; */
         break;
       case GST_RIFF_INFO_ITCH:
-        type = NULL; /*"Technician";*/
+        type = NULL;            /*"Technician"; */
         break;
       default:
         type = NULL;
         GST_WARNING ("Unknown INFO (metadata) tag entry " GST_FOURCC_FORMAT,
-		     GST_FOURCC_ARGS (tag));
+            GST_FOURCC_ARGS (tag));
         break;
     }
 
@@ -792,7 +848,10 @@ gst_riff_read_info (GstRiffRead *riff)
       }
 
       if (name && name[0] != '\0') {
-        GValue src = { 0 }, dest = { 0 };
+        GValue src = { 0 }
+        , dest =
+        {
+        0};
         GType dest_type = gst_tag_get_type (type);
 
         have_tags = TRUE;
@@ -802,7 +861,7 @@ gst_riff_read_info (GstRiffRead *riff)
         g_value_transform (&src, &dest);
         g_value_unset (&src);
         gst_tag_list_add_values (taglist, GST_TAG_MERGE_APPEND,
-				 type, &dest, NULL);
+            type, &dest, NULL);
         g_value_unset (&dest);
       }
       g_free (name);
@@ -818,16 +877,19 @@ gst_riff_read_info (GstRiffRead *riff)
 
     /* let the world know about this wonderful thing */
     for (padlist = gst_element_get_pad_list (element);
-	 padlist != NULL; padlist = padlist->next) {
-      if (GST_PAD_IS_SRC (padlist->data) && GST_PAD_IS_USABLE(padlist->data)) {
+        padlist != NULL; padlist = padlist->next) {
+      if (GST_PAD_IS_SRC (padlist->data) && GST_PAD_IS_USABLE (padlist->data)) {
         gst_event_ref (event);
         gst_pad_push (GST_PAD (padlist->data), GST_DATA (event));
       }
     }
+
+    gst_element_found_tags (element, taglist);
+
     gst_event_unref (event);
-    gst_element_found_tags (GST_ELEMENT (riff), taglist);
+  } else {
+    gst_tag_list_free (taglist);
   }
-  gst_tag_list_free (taglist);
 
   return TRUE;
 }
@@ -837,8 +899,7 @@ gst_riff_read_info (GstRiffRead *riff)
  */
 
 gboolean
-gst_riff_read_header (GstRiffRead *riff,
-		      guint32     *doctype)
+gst_riff_read_header (GstRiffRead * riff, guint32 * doctype)
 {
   GstRiffLevel *level;
   guint32 tag, length;
@@ -859,7 +920,7 @@ gst_riff_read_header (GstRiffRead *riff,
     return FALSE;
   }
   gst_bytestream_flush_fast (riff->bs, 4);
-  *doctype = GUINT32_FROM_LE (* (guint32 *) data);
+  *doctype = GST_READ_UINT32_LE (data);
 
   /* remember level */
   level = g_new (GstRiffLevel, 1);
