@@ -64,6 +64,10 @@ GType    gst_riff_read_get_type  (void);
 guint32  gst_riff_peek_tag       (GstRiffRead *riff,
 				  guint       *level_up);
 guint32  gst_riff_peek_list      (GstRiffRead *riff);
+gboolean gst_riff_peek_head      (GstRiffRead *riff,
+				  guint32     *tag,
+				  guint32     *length,
+				  guint       *level_up);
 
 GstEvent *gst_riff_read_seek      (GstRiffRead *riff,
 				  guint64      offset);
@@ -78,7 +82,9 @@ gboolean gst_riff_read_list      (GstRiffRead *riff,
 				  guint32     *tag);
 gboolean gst_riff_read_header    (GstRiffRead *read,
 				  guint32     *doctype);
-
+GstBuffer *gst_riff_read_element_data (GstRiffRead *riff,
+				       guint        length,
+				       guint       *got_bytes);
 /*
  * Utility functions (including byteswapping).
  */
@@ -86,6 +92,10 @@ gboolean gst_riff_read_strh      (GstRiffRead *riff,
 				  gst_riff_strh **header);
 gboolean gst_riff_read_strf_vids (GstRiffRead *riff,
 				  gst_riff_strf_vids **header);
+gboolean gst_riff_read_strf_vids_with_data
+				 (GstRiffRead *riff,
+				  gst_riff_strf_vids **header,
+				  GstBuffer  **extradata);
 gboolean gst_riff_read_strf_auds (GstRiffRead *riff,
 				  gst_riff_strf_auds **header);
 gboolean gst_riff_read_strf_iavs (GstRiffRead *riff,
