@@ -29,19 +29,19 @@ static GstElementClass *parent_class = NULL;
 /* Private methods */
 
 static void
-gst_videosink_set_clock (GstElement *element, GstClock *clock)
+gst_videosink_set_clock (GstElement * element, GstClock * clock)
 {
   GstVideoSink *videosink;
 
   videosink = GST_VIDEOSINK (element);
-  
+
   videosink->clock = clock;
 }
 
 /* Initing stuff */
 
 static void
-gst_videosink_init (GstVideoSink *videosink)
+gst_videosink_init (GstVideoSink * videosink)
 {
   videosink->width = 0;
   videosink->height = 0;
@@ -49,13 +49,13 @@ gst_videosink_init (GstVideoSink *videosink)
 }
 
 static void
-gst_videosink_class_init (GstVideoSinkClass *klass)
+gst_videosink_class_init (GstVideoSinkClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass*)klass;
-  gstelement_class = (GstElementClass*)klass;
+  gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
@@ -69,24 +69,22 @@ gst_videosink_get_type (void)
 {
   static GType videosink_type = 0;
 
-  if (!videosink_type)
-    {
-      static const GTypeInfo videosink_info = {
-        sizeof (GstVideoSinkClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) gst_videosink_class_init,
-        NULL,
-        NULL,
-        sizeof (GstVideoSink),
-        0,
-        (GInstanceInitFunc) gst_videosink_init,
-      };
-    
-      videosink_type = g_type_register_static (GST_TYPE_ELEMENT,
-                                               "GstVideoSink",
-                                               &videosink_info, 0);
-    }
-    
+  if (!videosink_type) {
+    static const GTypeInfo videosink_info = {
+      sizeof (GstVideoSinkClass),
+      NULL,
+      NULL,
+      (GClassInitFunc) gst_videosink_class_init,
+      NULL,
+      NULL,
+      sizeof (GstVideoSink),
+      0,
+      (GInstanceInitFunc) gst_videosink_init,
+    };
+
+    videosink_type = g_type_register_static (GST_TYPE_ELEMENT,
+        "GstVideoSink", &videosink_info, 0);
+  }
+
   return videosink_type;
 }
