@@ -62,7 +62,7 @@ static GstStaticPadTemplate gst_colorspace_src_template =
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_xRGB "; "
-        GST_VIDEO_CAPS_BGRx "; " GST_VIDEO_CAPS_RGB "; " GST_VIDEO_CAPS_RGB_16)
+        GST_VIDEO_CAPS_RGB "; " GST_VIDEO_CAPS_RGB_16)
     );
 
 /* Stereo signals and args */
@@ -76,7 +76,7 @@ enum
 {
   ARG_0,
   ARG_SOURCE,
-  ARG_DEST,
+  ARG_DEST
 };
 
 static void gst_colorspace_base_init (gpointer g_class);
@@ -263,7 +263,6 @@ gst_colorspace_caps_remove_format_info (GstCaps * caps, const char *media_type)
 {
   int i;
   GstStructure *structure;
-  GstCaps *rgbcaps;
 
   for (i = 0; i < gst_caps_get_size (caps); i++) {
     structure = gst_caps_get_structure (caps, i);
@@ -278,10 +277,8 @@ gst_colorspace_caps_remove_format_info (GstCaps * caps, const char *media_type)
     gst_structure_remove_field (structure, "blue_mask");
   }
 
-  rgbcaps = gst_caps_simplify (caps);
-  gst_caps_free (caps);
-
-  return rgbcaps;
+  gst_caps_do_simplify (caps);
+  return caps;
 }
 
 static GstCaps *
@@ -474,6 +471,7 @@ gst_colorspace_get_type (void)
       0,
       (GInstanceInitFunc) gst_colorspace_init,
     };
+
     colorspace_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstColorspace",
         &colorspace_info, 0);
@@ -620,7 +618,7 @@ gst_colorspace_get_property (GObject * object, guint prop_id, GValue * value,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "colorspace", GST_RANK_PRIMARY,
+  if (!gst_element_register (plugin, "colorspace", GST_RANK_PRIMARY - 1,
           GST_TYPE_COLORSPACE))
     return FALSE;
 
