@@ -23,35 +23,16 @@
 #include <avcodec.h>
 #include <gst/gst.h>
 
-/*
- * _codectype_to_caps () gets the GstCaps that belongs to
- * a certain CodecType for a pad with uncompressed data.
- */
+/* Template caps */
 
 GstCaps *
-gst_ffmpegcsp_codectype_to_caps (enum CodecType  codec_type,
-                                 AVCodecContext *context);
+gst_ffmpeg_pix_fmt_to_caps (void);
 
-/*
- * caps_with_codectype () transforms a GstCaps that belongs to
- * a pad for uncompressed data to a filled-in context.
- */
+/* Disect a GstCaps */
 
-void
-gst_ffmpegcsp_caps_with_codectype (enum CodecType  type,
-                                   const GstCaps  *caps,
-                                   AVCodecContext *context);
-
-/*
- * Fill in pointers in an AVPicture, aligned by 4 (required by X).
- */
-
-int
-gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
-			      uint8_t *   ptr,
-			      enum PixelFormat pix_fmt,
-			      int         width,
-			      int         height);
+enum PixelFormat
+gst_ffmpeg_caps_to_pix_fmt (const GstCaps *caps,
+			    int *width, int *height,
+			    double *fps);
 
 #endif /* __GST_FFMPEG_CODECMAP_H__ */
-
