@@ -45,7 +45,6 @@
 #define GST_MATROSKA_ID_WRITINGAPP 0x5741
 #define GST_MATROSKA_ID_MUXINGAPP  0x4D80
 #define GST_MATROSKA_ID_DATEUTC    0x4461
-#define GST_MATROSKA_ID_SEGMENTUID 0x73A4
 
 /* ID in the tracks master */
 #define GST_MATROSKA_ID_TRACKENTRY 0xAE
@@ -117,7 +116,6 @@
 /* IDs in the cluster master */
 #define GST_MATROSKA_ID_CLUSTERTIMECODE 0xE7
 #define GST_MATROSKA_ID_BLOCKGROUP 0xA0
-#define GST_MATROSKA_ID_REFERENCEBLOCK 0xFB
 
 /* IDs in the blockgroup master */
 #define GST_MATROSKA_ID_BLOCK      0xA1
@@ -150,11 +148,8 @@
 #define GST_MATROSKA_CODEC_ID_AUDIO_ACM          "A_MS/ACM"
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG2        "A_AAC/MPEG2/"
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG4        "A_AAC/MPEG4/"
-#define GST_MATROSKA_CODEC_ID_AUDIO_TTA          "A_TTA1"
 /* TODO: AC3-9/10 (?), Real, Musepack, Quicktime */
 
-#define GST_MATROSKA_CODEC_ID_SUBTITLE_UTF8      "S_TEXT/UTF8"
-
 /*
  * Matrodka tags. Strings.
  */
@@ -225,7 +220,6 @@ typedef struct _GstMatroskaTrackContext {
   guint         uid, num;
   GstMatroskaTrackFlags flags;
   guint64       default_duration;
-  guint64       pos;
 } GstMatroskaTrackContext;
 
 typedef struct _GstMatroskaTrackVideoContext {
@@ -242,11 +236,6 @@ typedef struct _GstMatroskaTrackAudioContext {
   GstMatroskaTrackContext parent;
 
   guint         samplerate, channels, bitdepth;
-  
-  /* Special flag for Vorbis, we need to send codec_priv first before
-   * sending any data, and just testing for time == 0 is not enough
-   * to detect that */
-  gboolean 	first_frame;
 } GstMatroskaTrackAudioContext;
 
 typedef struct _GstMatroskaTrackComplexContext {
