@@ -42,31 +42,12 @@ G_BEGIN_DECLS
 
 #define GST_MATROSKA_MUX_MAX_STREAMS 64
 
-typedef struct _BITMAPINFOHEADER {
-  guint32 bi_size;
-  guint32 bi_width;
-  guint32 bi_height;
-  guint16 bi_planes;
-  guint16 bi_bit_count;
-  guint32 bi_compression;
-  guint32 bi_size_image;
-  guint32 bi_x_pels_per_meter;
-  guint32 bi_y_pels_per_meter;
-  guint32 bi_clr_used;
-  guint32 bi_clr_important;
-} BITMAPINFOHEADER;
-
 typedef enum {
   GST_MATROSKA_MUX_STATE_START,
   GST_MATROSKA_MUX_STATE_HEADER,
   GST_MATROSKA_MUX_STATE_DATA,
 } GstMatroskaMuxState;
 
-typedef struct _GstMatroskaMetaSeekIndex {
-  guint32  id;
-  guint64  pos;
-} GstMatroskaMetaSeekIndex;
-
 typedef struct _GstMatroskaMux {
   GstEbmlWrite   parent;
 
@@ -76,7 +57,6 @@ typedef struct _GstMatroskaMux {
     GstMatroskaTrackContext *track;
     GstBuffer   *buffer;
     gboolean     eos;
-    guint64      duration;
   } sink[GST_MATROSKA_MUX_MAX_STREAMS];
   guint          num_streams,
                  num_v_streams, num_a_streams, num_t_streams;
@@ -106,18 +86,8 @@ typedef struct _GstMatroskaMux {
 #endif
 		 info_pos,
 		 tracks_pos,
-		 duration_pos,
-     meta_pos;
+		 duration_pos;
   guint64        segment_master;
-
-  /* current cluster */
-  guint64        cluster,
-                 cluster_time,
-                 cluster_pos;
-
-  /* meta-seek info */
-  GstMatroskaMetaSeekIndex *meta_index;
-  guint          num_meta_indexes;
 } GstMatroskaMux;
 
 typedef struct _GstMatroskaMuxClass {
