@@ -22,7 +22,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include "gstmpeg2subt.h"
+#include <gstmpeg2subt.h>
 #include <string.h>
 
 static void gst_mpeg2subt_class_init (GstMpeg2SubtClass * klass);
@@ -68,13 +68,6 @@ static GstStaticPadTemplate video_template = GST_STATIC_PAD_TEMPLATE ("video",
         "width = (int) [ 16, 4096 ], " "height = (int) [ 16, 4096 ]")
     );
 
-static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
-    GST_PAD_SRC,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw-yuv, " "format = (fourcc) { I420 }, " /* YV12 later */
-        "width = (int) [ 16, 4096 ], " "height = (int) [ 16, 4096 ]")
-    );
-
 static GstStaticPadTemplate subtitle_template =
 GST_STATIC_PAD_TEMPLATE ("subtitle",
     GST_PAD_SINK,
@@ -112,13 +105,6 @@ enum
   SPU_END = 0xff
 };
 
-static guint32 default_clut[16] = {
-  0xb48080, 0x248080, 0x628080, 0xd78080,
-  0x808080, 0x808080, 0x808080, 0x808080,
-  0x808080, 0x808080, 0x808080, 0x808080,
-  0x808080, 0x808080, 0x808080, 0x808080
-};
-
 typedef struct RLE_state
 {
   gint id;
@@ -174,8 +160,6 @@ gst_mpeg2subt_base_init (GstMpeg2SubtClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&video_template));
   gst_element_class_add_pad_template (element_class,
@@ -219,9 +203,7 @@ gst_mpeg2subt_init (GstMpeg2Subt * mpeg2subt)
       (&subtitle_template), "subtitle");
   gst_element_add_pad (GST_ELEMENT (mpeg2subt), mpeg2subt->subtitlepad);
 
-  mpeg2subt->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&src_template), "src");
+  mpeg2subt->srcpad = gst_pad_new ("src", GST_PAD_SRC);
   gst_element_add_pad (GST_ELEMENT (mpeg2subt), mpeg2subt->srcpad);
   gst_pad_set_getcaps_function (mpeg2subt->srcpad,
       GST_DEBUG_FUNCPTR (gst_mpeg2subt_getcaps_video));
@@ -240,12 +222,11 @@ gst_mpeg2subt_init (GstMpeg2Subt * mpeg2subt)
   mpeg2subt->start_display_time = GST_CLOCK_TIME_NONE;
   mpeg2subt->end_display_time = GST_CLOCK_TIME_NONE;
   mpeg2subt->forced_display = FALSE;
-
-  memcpy (mpeg2subt->current_clut, default_clut, sizeof (guint32) * 16);
-  gst_setup_palette (mpeg2subt, mpeg2subt->menu_index, mpeg2subt->menu_alpha);
-  gst_setup_palette (mpeg2subt, mpeg2subt->subtitle_index,
-      mpeg2subt->subtitle_alpha);
-
+  memset (mpeg2subt->current_clut, 0, 16 * sizeof (guint32));
+  memset (mpeg2subt->subtitle_index, 0, sizeof (mpeg2subt->subtitle_index));
+  memset (mpeg2subt->menu_index, 0, sizeof (mpeg2subt->menu_index));
+  memset (mpeg2subt->subtitle_alpha, 0, sizeof (mpeg2subt->subtitle_alpha));
+  memset (mpeg2subt->menu_alpha, 0, sizeof (mpeg2subt->menu_alpha));
   memset (mpeg2subt->out_buffers, 0, sizeof (mpeg2subt->out_buffers));
   mpeg2subt->pending_video_buffer = NULL;
   mpeg2subt->next_video_time = GST_CLOCK_TIME_NONE;
@@ -521,7 +502,7 @@ gst_mpeg2subt_parse_header (GstMpeg2Subt * mpeg2subt)
         mpeg2subt->subtitle_alpha);
 }
 
-static inline int
+inline int
 gst_get_nibble (guchar * buffer, RLE_state * state)
 {
   if (state->aligned) {
@@ -551,7 +532,7 @@ gst_setup_palette (GstMpeg2Subt * mpeg2subt, guchar * indexes, guchar * alpha)
   }
 }
 
-static inline guint
+inline guint
 gst_get_rle_code (guchar * buffer, RLE_state * state)
 {
   gint code;
@@ -635,7 +616,7 @@ gst_draw_rle_line (GstMpeg2Subt * mpeg2subt, guchar * buffer, RLE_state * state)
   }
 }
 
-static inline void
+inline void
 gst_merge_uv_data (GstMpeg2Subt * mpeg2subt, guchar * buffer, RLE_state * state)
 {
   gint x;
@@ -704,7 +685,6 @@ gst_mpeg2subt_merge_title (GstMpeg2Subt * mpeg2subt, GstBuffer * buf)
       " using %s colour table", GST_BUFFER_TIMESTAMP (buf),
       mpeg2subt->forced_display ? "menu" : "subtitle");
 
-  state.next = 0;
   state.id = 0;
   state.aligned = 1;
   state.offset[0] = mpeg2subt->offset[0];
@@ -909,7 +889,7 @@ gst_mpeg2subt_handle_dvd_event (GstMpeg2Subt * mpeg2subt, GstEvent * event,
     gst_update_still_frame (mpeg2subt);
   } else if (from_sub_pad && !strcmp (event_type, "dvd-spu-clut-change")) {
     /* Take a copy of the colour table */
-    gchar name[16];
+    guchar name[16];
     int i;
     gint value;
 
