@@ -24,17 +24,10 @@
 #endif
 #include <string.h>
 
-#include "gstwavparse.h"
+#include <gstwavparse.h>
 #include "gst/riff/riff-ids.h"
 #include "gst/riff/riff-media.h"
 
-#ifndef G_MAXUINT32
-#define G_MAXUINT32 0xffffffff
-#endif
-
-GST_DEBUG_CATEGORY (wavparse_debug);
-#define GST_CAT_DEFAULT (wavparse_debug)
-
 static void gst_wavparse_base_init (gpointer g_class);
 static void gst_wavparse_class_init (GstWavParseClass * klass);
 static void gst_wavparse_init (GstWavParse * wavparse);
@@ -69,8 +62,8 @@ static GstStaticPadTemplate src_template_factory =
     GST_STATIC_CAPS ("audio/x-raw-int, "
         "endianness = (int) little_endian, "
         "signed = (boolean) { true, false }, "
-        "width = (int) { 8, 16, 24, 32 }, "
-        "depth = (int) { 8, 16, 24, 32 }, "
+        "width = (int) { 8, 16 }, "
+        "depth = (int) { 8, 16 }, "
         "rate = (int) [ 8000, 48000 ], "
         "channels = (int) [ 1, 2 ]; "
         "audio/mpeg, "
@@ -91,8 +84,7 @@ static GstStaticPadTemplate src_template_factory =
         "audio/x-adpcm, "
         "layout = (string) dvi, "
         "block_align = (int) [ 1, 8192 ], "
-        "rate = (int) [ 8000, 48000 ], " "channels = (int) [ 1, 2 ];"
-        "audio/x-vnd.sony.atrac3")
+        "rate = (int) [ 8000, 48000 ], " "channels = (int) [ 1, 2 ]")
     );
 
 /* WavParse signals and args */
@@ -169,8 +161,6 @@ gst_wavparse_class_init (GstWavParseClass * klass)
 
   object_class->get_property = gst_wavparse_get_property;
   gstelement_class->change_state = gst_wavparse_change_state;
-
-  GST_DEBUG_CATEGORY_INIT (wavparse_debug, "wavparse", 0, "WAV parser");
 }
 
 static void
@@ -229,7 +219,7 @@ gst_wavparse_destroy_sourcepad (GstWavParse * wavparse)
   }
 }
 
-static gboolean
+static void
 gst_wavparse_create_sourcepad (GstWavParse * wavparse)
 {
   gst_wavparse_destroy_sourcepad (wavparse);
@@ -238,13 +228,6 @@ gst_wavparse_create_sourcepad (GstWavParse * wavparse)
   wavparse->srcpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&src_template_factory), "src");
-
-  if (!wavparse->srcpad) {
-    GST_ELEMENT_ERROR (wavparse, CORE, PAD,
-        ("Failed to allocate new pad from template"), (NULL));
-    return FALSE;
-  }
-
   gst_pad_use_explicit_caps (wavparse->srcpad);
   gst_pad_set_formats_function (wavparse->srcpad, gst_wavparse_get_formats);
   gst_pad_set_convert_function (wavparse->srcpad, gst_wavparse_pad_convert);
@@ -254,8 +237,6 @@ gst_wavparse_create_sourcepad (GstWavParse * wavparse)
   gst_pad_set_event_function (wavparse->srcpad, gst_wavparse_srcpad_event);
   gst_pad_set_event_mask_function (wavparse->srcpad,
       gst_wavparse_get_event_masks);
-
-  return TRUE;
 }
 
 static void
@@ -528,10 +509,8 @@ gst_wavparse_stream_init (GstWavParse * wav)
   GstRiffRead *riff = GST_RIFF_READ (wav);
   guint32 doctype;
 
-  if (!gst_riff_read_header (riff, &doctype)) {
-    GST_WARNING_OBJECT (wav, "could not read header");
+  if (!gst_riff_read_header (riff, &doctype))
     return FALSE;
-  }
 
   if (doctype != GST_RIFF_RIFF_WAVE) {
     GST_ELEMENT_ERROR (wav, STREAM, WRONG_TYPE, (NULL), (NULL));
@@ -546,7 +525,7 @@ static gboolean
 gst_wavparse_fmt (GstWavParse * wav)
 {
   GstRiffRead *riff = GST_RIFF_READ (wav);
-  gst_riff_strf_auds *header = NULL;
+  gst_riff_strf_auds *header;
   GstCaps *caps;
 
   if (!gst_riff_read_strf_auds (riff, &header)) {
@@ -554,54 +533,24 @@ gst_wavparse_fmt (GstWavParse * wav)
     return FALSE;
   }
 
+  gst_wavparse_create_sourcepad (wav);
+
   wav->format = header->format;
   wav->rate = header->rate;
   wav->channels = header->channels;
-  if (wav->channels == 0) {
-    GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
-        ("Stream claims to contain zero channels - invalid data"));
-    g_free (header);
-    return FALSE;
-  }
-  wav->blockalign = header->blockalign;
   wav->width = (header->blockalign * 8) / header->channels;
   wav->depth = header->size;
   wav->bps = header->av_bps;
-  if (wav->bps <= 0) {
-    GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
-        ("Stream claims to bitrate of <= zero - invalid data"));
-    g_free (header);
-    return FALSE;
-  }
 
   caps = gst_riff_create_audio_caps (header->format, NULL, header, NULL);
-  g_free (header);
 
   if (caps) {
-    if (!gst_wavparse_create_sourcepad (wav))
-      return FALSE;
-
-    if (!gst_pad_set_explicit_caps (wav->srcpad, caps)) {
-      gchar *caps_str = gst_caps_to_string (caps);
-
-      GST_ELEMENT_ERROR (wav, CORE, PAD, (NULL),
-          ("failed to negotiate (try_set_caps with \"%s\" returned REFUSED)",
-              caps_str));
-      g_free (caps_str);
-      gst_caps_free (caps);
-      return FALSE;
-    }
-
+    gst_pad_set_explicit_caps (wav->srcpad, caps);
     gst_caps_free (caps);
-    gst_element_add_pad (GST_ELEMENT (wav), wav->srcpad);
-    gst_element_no_more_pads (GST_ELEMENT (wav));
-    GST_DEBUG ("frequency %d, channels %d", wav->rate, wav->channels);
-  } else {
-    GST_ELEMENT_ERROR (wav, STREAM, TYPE_NOT_FOUND, (NULL),
-        ("No caps found for format 0x%x, %d channels, %d Hz",
-            wav->format, wav->channels, wav->rate));
-    return FALSE;
   }
+  gst_element_add_pad (GST_ELEMENT (wav), wav->srcpad);
+
+  GST_DEBUG ("frequency %d, channels %d", wav->rate, wav->channels);
 
   return TRUE;
 }
@@ -613,93 +562,49 @@ gst_wavparse_other (GstWavParse * wav)
   guint32 tag, length;
 
   if (!gst_riff_peek_head (riff, &tag, &length, NULL)) {
-    GST_WARNING_OBJECT (wav, "could not peek head");
     return FALSE;
   }
-  GST_DEBUG_OBJECT (wav, "got tag (%08x) %4.4s, length %d", tag,
-      (gchar *) & tag, length);
 
   switch (tag) {
     case GST_RIFF_TAG_LIST:
       if (!(tag = gst_riff_peek_list (riff))) {
-        GST_WARNING_OBJECT (wav, "could not peek list");
         return FALSE;
       }
 
       switch (tag) {
         case GST_RIFF_LIST_INFO:
-          if (!gst_riff_read_list (riff, &tag) || !gst_riff_read_info (riff)) {
-            GST_WARNING_OBJECT (wav, "could not read list");
+          if (!gst_riff_read_list (riff, &tag) || !gst_riff_read_info (riff))
             return FALSE;
-          }
           break;
 
         case GST_RIFF_LIST_adtl:
-          if (!gst_riff_read_skip (riff)) {
-            GST_WARNING_OBJECT (wav, "could not read skip");
+          if (!gst_riff_read_skip (riff))
             return FALSE;
-          }
           break;
 
         default:
-          GST_DEBUG_OBJECT (wav, "skipping tag (%08x) %4.4s", tag,
-              (gchar *) & tag);
-          if (!gst_riff_read_skip (riff)) {
-            GST_WARNING_OBJECT (wav, "could not read skip");
+          if (!gst_riff_read_skip (riff))
             return FALSE;
-          }
           break;
       }
 
       break;
 
     case GST_RIFF_TAG_data:
-      if (!gst_bytestream_flush (riff->bs, 8)) {
-        GST_WARNING_OBJECT (wav, "could not flush 8 bytes");
+      if (!gst_bytestream_flush (riff->bs, 8))
         return FALSE;
-      }
 
-      GST_DEBUG_OBJECT (wav, "switching to data mode");
       wav->state = GST_WAVPARSE_DATA;
-      wav->datastart = gst_bytestream_tell (riff->bs);
-      if (length == 0) {
-        guint64 file_length;
-
-        /* length is 0, data probably stretches to the end
-         * of file */
-        GST_DEBUG_OBJECT (wav, "length is 0 trying to find length");
-        /* get length of file */
-        file_length = gst_bytestream_length (riff->bs);
-        if (file_length == -1) {
-          GST_DEBUG_OBJECT (wav,
-              "could not get file length, assuming data to eof");
-          /* could not get length, assuming till eof */
-          length = G_MAXUINT32;
-        }
-        if (file_length > G_MAXUINT32) {
-          GST_DEBUG_OBJECT (wav, "file length %lld, clipping to 32 bits");
-          /* could not get length, assuming till eof */
-          length = G_MAXUINT32;
-        } else {
-          GST_DEBUG_OBJECT (wav, "file length %lld, datalength", file_length,
-              length);
-          /* substract offset of datastart from length */
-          length = file_length - wav->datastart;
-          GST_DEBUG_OBJECT (wav, "datalength %lld", length);
-        }
-      }
       wav->dataleft = wav->datasize = (guint64) length;
+      wav->datastart = gst_bytestream_tell (riff->bs);
       break;
 
     case GST_RIFF_TAG_cue:
-      if (!gst_riff_read_skip (riff)) {
-        GST_WARNING_OBJECT (wav, "could not read skip");
+      if (!gst_riff_read_skip (riff))
         return FALSE;
-      }
       break;
 
     default:
-      GST_DEBUG_OBJECT (wav, "skipping tag (%08x) %4.4s", tag, (gchar *) & tag);
       if (!gst_riff_read_skip (riff))
         return FALSE;
       break;
@@ -711,6 +616,7 @@ gst_wavparse_other (GstWavParse * wav)
 static gboolean
 gst_wavparse_handle_seek (GstWavParse * wav)
 {
+#if 1
   GstRiffRead *riff = GST_RIFF_READ (wav);
   GstEvent *event = NULL;
   guint32 remaining;
@@ -753,6 +659,9 @@ gst_wavparse_handle_seek (GstWavParse * wav)
   gst_pad_event_default (wav->sinkpad, event);
 
   return TRUE;
+#else
+  return FALSE;
+#endif
 }
 
 #define MAX_BUFFER_SIZE 4096
@@ -775,14 +684,8 @@ gst_wavparse_loop (GstElement * element)
       GstBuffer *buf = NULL;
 
       desired = MIN (wav->dataleft, MAX_BUFFER_SIZE);
-      if (desired >= wav->blockalign)
-        desired -= (desired % wav->blockalign);
-      if (!(buf = gst_riff_read_element_data (riff, desired, &got_bytes))) {
-        GST_WARNING_OBJECT (wav, "trying to read %d bytes failed", desired);
+      if (!(buf = gst_riff_read_element_data (riff, desired, &got_bytes)))
         return;
-      }
-      GST_DEBUG_OBJECT (wav, "read %d bytes, got %d bytes", desired, got_bytes);
-
       GST_BUFFER_TIMESTAMP (buf) = GST_SECOND *
           (wav->datasize - wav->dataleft) / wav->bps;
       GST_BUFFER_DURATION (buf) = GST_SECOND * got_bytes / wav->bps;
@@ -901,9 +804,7 @@ gst_wavparse_pad_convert (GstPad * pad,
     case GST_FORMAT_TIME:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
-          /* make sure we end up on a sample boundary */
-          *dest_value =
-              (src_value * wavparse->rate / GST_SECOND) * wavparse->blockalign;
+          *dest_value = src_value * byterate / GST_SECOND;
           break;
         case GST_FORMAT_DEFAULT:
           *dest_value = src_value * wavparse->rate / GST_SECOND;
@@ -960,8 +861,8 @@ gst_wavparse_pad_query (GstPad * pad, GstQueryType type,
     return TRUE;
   }
 
-  return gst_pad_convert (wav->sinkpad, GST_FORMAT_BYTES,
-      bytevalue, format, value);
+  return gst_pad_convert (wav->sinkpad, bytevalue,
+      GST_FORMAT_BYTES, format, value);
 }
 
 static const GstEventMask *
@@ -978,6 +879,7 @@ gst_wavparse_get_event_masks (GstPad * pad)
 static gboolean
 gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event)
 {
+#if 1
   GstWavParse *wavparse = GST_WAVPARSE (GST_PAD_PARENT (pad));
   gboolean res = FALSE;
 
@@ -989,7 +891,8 @@ gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event)
       gint64 byteoffset;
       GstFormat format;
 
-      /* bring format to samples for the peer element, */
+      /* bring format to bytes for the peer element, 
+       * FIXME be smarter here */
       format = GST_FORMAT_BYTES;
       res = gst_pad_convert (pad,
           GST_EVENT_SEEK_FORMAT (event),
@@ -1009,6 +912,9 @@ gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event)
   gst_event_unref (event);
 
   return res;
+#else
+  return FALSE;
+#endif
 }
 
 static GstElementStateReturn
@@ -1060,7 +966,7 @@ plugin_init (GstPlugin * plugin)
     return FALSE;
   }
 
-  return gst_element_register (plugin, "wavparse", GST_RANK_PRIMARY,
+  return gst_element_register (plugin, "wavparse", GST_RANK_SECONDARY,
       GST_TYPE_WAVPARSE);
 }
 
