@@ -75,15 +75,10 @@ static GstStaticPadTemplate osssrc_src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
-        "endianness = (int) { LITTLE_ENDIAN, BIG_ENDIAN }, "
+        "endianness = (int) BYTE_ORDER, "
         "signed = (boolean) { TRUE, FALSE }, "
-        "width = (int) 16, "
+        "width = (int) { 8, 16 }, "
         "depth = (int) { 8, 16 }, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]; "
-        "audio/x-raw-int, "
-        "signed = (boolean) { TRUE, FALSE }, "
-        "width = (int) 8, "
-        "depth = (int) 8, "
         "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]")
     );
 
@@ -222,10 +217,7 @@ gst_osssrc_dispose (GObject * object)
 {
   GstOssSrc *osssrc = (GstOssSrc *) object;
 
-  if (osssrc->provided_clock != NULL) {
-    gst_object_unparent (GST_OBJECT (osssrc->provided_clock));
-    osssrc->provided_clock = NULL;
-  }
+  gst_object_unparent (GST_OBJECT (osssrc->provided_clock));
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -308,7 +300,7 @@ gst_osssrc_get_time (GstClock * clock, gpointer data)
   if (ioctl (GST_OSSELEMENT (osssrc)->fd, SNDCTL_DSP_GETISPACE, &info) < 0)
     return 0;
 
-  return (osssrc->curoffset * GST_OSSELEMENT (osssrc)->sample_width +
+  return (osssrc->curoffset +
       info.bytes) * GST_SECOND / GST_OSSELEMENT (osssrc)->bps;
 }
 
@@ -338,7 +330,6 @@ gst_osssrc_get (GstPad * pad)
   GstOssSrc *src;
   GstBuffer *buf;
   glong readbytes;
-  glong readsamples;
 
   src = GST_OSSSRC (gst_pad_get_parent (pad));
 
@@ -380,21 +371,19 @@ gst_osssrc_get (GstPad * pad)
     return GST_DATA (gst_event_new (GST_EVENT_INTERRUPT));
   }
 
-  readsamples = readbytes * GST_OSSELEMENT (src)->rate /
-      GST_OSSELEMENT (src)->bps;
-
   GST_BUFFER_SIZE (buf) = readbytes;
   GST_BUFFER_OFFSET (buf) = src->curoffset;
-  GST_BUFFER_OFFSET_END (buf) = src->curoffset + readsamples;
+
   GST_BUFFER_DURATION (buf) =
-      readsamples * GST_SECOND / GST_OSSELEMENT (src)->rate;
+      (GST_SECOND * GST_BUFFER_SIZE (buf)) / (GST_OSSELEMENT (src)->bps *
+      GST_OSSELEMENT (src)->rate);
 
   /* if we have a clock */
   if (src->clock) {
     if (src->clock == src->provided_clock) {
       /* if it's our own clock, we can be very accurate */
       GST_BUFFER_TIMESTAMP (buf) =
-          src->curoffset * GST_SECOND / GST_OSSELEMENT (src)->rate;
+          src->curoffset * GST_SECOND / GST_OSSELEMENT (src)->bps;
     } else {
       /* somebody elses clock, timestamp with that clock, no discontinuity in
        * the stream since the OFFSET is updated correctly. Elements can stretch
@@ -407,7 +396,7 @@ gst_osssrc_get (GstPad * pad)
     GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
   }
 
-  src->curoffset += readsamples;
+  src->curoffset += readbytes;
 
   GST_DEBUG ("pushed buffer from soundcard of %ld bytes, timestamp %"
       G_GINT64_FORMAT, readbytes, GST_BUFFER_TIMESTAMP (buf));
@@ -594,7 +583,7 @@ gst_osssrc_src_query (GstPad * pad, GstQueryType type, GstFormat * format,
   switch (type) {
     case GST_QUERY_POSITION:
       res = gst_osselement_convert (GST_OSSELEMENT (osssrc),
-          GST_FORMAT_DEFAULT, osssrc->curoffset, format, value);
+          GST_FORMAT_BYTES, osssrc->curoffset, format, value);
       break;
     default:
       break;
